@@ -1,7 +1,8 @@
 /* A voice removal plugin 
-   by Anders Carlsson <anders.carlsson@tordata.se> */
+   by Anders Carlsson <andersca@gnu.org> */
 
 #include "voice.h"
+#include "xmms/i18n.h"
 
 static int mod_samples(gpointer * d, gint length, AFormat afmt, gint srate, gint nch);
 
@@ -9,7 +10,7 @@ EffectPlugin voice_ep =
 {
 	NULL,
 	NULL,
-	"Voice removal plugin " VERSION,
+	NULL, /* Description */
 	NULL,
 	NULL,
 	voice_about,
@@ -19,6 +20,7 @@ EffectPlugin voice_ep =
 
 EffectPlugin *get_eplugin_info(void)
 {
+	voice_ep.description = g_strdup_printf(_("Voice removal plugin %s"), VERSION);
 	return &voice_ep;
 }
 
