@@ -1,7 +1,5 @@
-
 #include "mpg123.h"
 #include "libxmms/configfile.h"
-#include "id3.h"
 #include <string.h>
 #include <stdlib.h>
 #include <pthread.h>
@@ -20,36 +18,60 @@ static gboolean audio_error = FALSE;
 gint mpg123_bitrate, mpg123_frequency, mpg123_length, mpg123_layer, mpg123_lsf;
 gchar *mpg123_title = NULL, *mpg123_filename = NULL;
 static gint disp_bitrate, skip_frames = 0;
-gboolean mpg123_stereo;
+gboolean mpg123_stereo, mpg123_mpeg25;
+gint mpg123_mode;
+#ifdef USE_3DNOW
+static void aboutbox(void);
+static GtkWidget *about_window = NULL;
+#else
+#define aboutbox NULL
+#endif
 
 const gchar *mpg123_id3_genres[GENRE_MAX] =
 {
-	"Blues", "Classic Rock", "Country", "Dance", "Disco", "Funk",
-	"Grunge", "Hip-Hop", "Jazz", "Metal", "New Age", "Oldies", "Other",
-	"Pop", "R&B", "Rap", "Reggae", "Rock", "Techno", "Industrial",
-	"Alternative", "Ska", "Death Metal", "Pranks", "Soundtrack",
-	"Euro-Techno", "Ambient", "Trip-Hop", "Vocal", "Jazz+Funk", "Fusion",
-	"Trance", "Classical", "Instrumental", "Acid", "House", "Game",
-	"Sound Clip", "Gospel", "Noise", "Alt", "Bass", "Soul", "Punk", "Space",
-	"Meditative", "Instrumental Pop", "Instrumental Rock", "Ethnic",
-	"Gothic", "Darkwave", "Techno-Industrial", "Electronic", "Pop-Folk",
-	"Eurodance", "Dream", "Southern Rock", "Comedy", "Cult", "Gangsta Rap",
-	"Top 40", "Christian Rap", "Pop/Funk", "Jungle", "Native American",
-	"Cabaret", "New Wave", "Psychedelic", "Rave", "Showtunes", "Trailer",
-	"Lo-Fi", "Tribal", "Acid Punk", "Acid Jazz", "Polka", "Retro", "Musical",
-	"Rock & Roll", "Hard Rock", "Folk", "Folk/Rock", "National Folk", "Swing",
-	"Fast-Fusion", "Bebob", "Latin", "Revival", "Celtic", "Bluegrass",
-	"Avantgarde", "Gothic Rock", "Progressive Rock", "Psychedelic Rock",
-	"Symphonic Rock", "Slow Rock", "Big Band", "Chorus", "Easy Listening",
-	"Acoustic", "Humour", "Speech", "Chanson", "Opera", "Chamber Music",
-	"Sonata", "Symphony", "Booty Bass", "Primus", "Porn Groove", "Satire",
-	"Slow Jam", "Club", "Tango", "Samba", "Folklore", "Ballad", "Power Ballad",
-	"Rhythmic Soul", "Freestyle", "Duet", "Punk Rock", "Drum Solo", "A Cappella",
-	"Euro-House", "Dance Hall", "Goa", "Drum & Bass", "Club-House", "Hardcore",
-	"Terror", "Indie", "BritPop", "Negerpunk", "Polsk Punk", "Beat",
-	"Christian Gangsta Rap", "Heavy Metal", "Black Metal", "Crossover",
-	"Contemporary Christian", "Christian Rock", "Merengue", "Salsa",
-	"Thrash Metal", "Anime", "JPop", "Synthpop"
+	N_("Blues"), N_("Classic Rock"), N_("Country"), N_("Dance"),
+	N_("Disco"), N_("Funk"), N_("Grunge"), N_("Hip-Hop"),
+	N_("Jazz"), N_("Metal"), N_("New Age"), N_("Oldies"),
+	N_("Other"), N_("Pop"), N_("R&B"), N_("Rap"), N_("Reggae"),
+	N_("Rock"), N_("Techno"), N_("Industrial"), N_("Alternative"),
+	N_("Ska"), N_("Death Metal"), N_("Pranks"), N_("Soundtrack"),
+	N_("Euro-Techno"), N_("Ambient"), N_("Trip-Hop"), N_("Vocal"),
+	N_("Jazz+Funk"), N_("Fusion"), N_("Trance"), N_("Classical"),
+	N_("Instrumental"), N_("Acid"), N_("House"), N_("Game"),
+	N_("Sound Clip"), N_("Gospel"), N_("Noise"), N_("Alt"),
+	N_("Bass"), N_("Soul"), N_("Punk"), N_("Space"),
+	N_("Meditative"), N_("Instrumental Pop"),
+	N_("Instrumental Rock"), N_("Ethnic"), N_("Gothic"),
+	N_("Darkwave"), N_("Techno-Industrial"), N_("Electronic"),
+	N_("Pop-Folk"), N_("Eurodance"), N_("Dream"),
+	N_("Southern Rock"), N_("Comedy"), N_("Cult"),
+	N_("Gangsta Rap"), N_("Top 40"), N_("Christian Rap"),
+	N_("Pop/Funk"), N_("Jungle"), N_("Native American"),
+	N_("Cabaret"), N_("New Wave"), N_("Psychedelic"), N_("Rave"),
+	N_("Showtunes"), N_("Trailer"), N_("Lo-Fi"), N_("Tribal"),
+	N_("Acid Punk"), N_("Acid Jazz"), N_("Polka"), N_("Retro"),
+	N_("Musical"), N_("Rock & Roll"), N_("Hard Rock"), N_("Folk"),
+	N_("Folk/Rock"), N_("National Folk"), N_("Swing"),
+	N_("Fast-Fusion"), N_("Bebob"), N_("Latin"), N_("Revival"),
+	N_("Celtic"), N_("Bluegrass"), N_("Avantgarde"),
+	N_("Gothic Rock"), N_("Progressive Rock"),
+	N_("Psychedelic Rock"), N_("Symphonic Rock"), N_("Slow Rock"),
+	N_("Big Band"), N_("Chorus"), N_("Easy Listening"),
+	N_("Acoustic"), N_("Humour"), N_("Speech"), N_("Chanson"),
+	N_("Opera"), N_("Chamber Music"), N_("Sonata"), N_("Symphony"),
+	N_("Booty Bass"), N_("Primus"), N_("Porn Groove"),
+	N_("Satire"), N_("Slow Jam"), N_("Club"), N_("Tango"),
+	N_("Samba"), N_("Folklore"), N_("Ballad"), N_("Power Ballad"),
+	N_("Rhythmic Soul"), N_("Freestyle"), N_("Duet"),
+	N_("Punk Rock"), N_("Drum Solo"), N_("A Cappella"),
+	N_("Euro-House"), N_("Dance Hall"), N_("Goa"),
+	N_("Drum & Bass"), N_("Club-House"), N_("Hardcore"),
+	N_("Terror"), N_("Indie"), N_("BritPop"), N_("Negerpunk"),
+	N_("Polsk Punk"), N_("Beat"), N_("Christian Gangsta Rap"),
+	N_("Heavy Metal"), N_("Black Metal"), N_("Crossover"),
+	N_("Contemporary Christian"), N_("Christian Rock"),
+	N_("Merengue"), N_("Salsa"), N_("Thrash Metal"),
+	N_("Anime"), N_("JPop"), N_("Synthpop")
 };
 
 double mpg123_compute_tpf(struct frame *fr)
@@ -67,11 +89,19 @@ void set_mpg123_synth_functions(struct frame *fr)
 {
 	typedef int (*func) (real *, int, unsigned char *, int *);
 	typedef int (*func_mono) (real *, unsigned char *, int *);
+#ifdef USE_3DNOW
+	typedef void (*func_dct36)(real *,real *,real *,real *,real *);
+#endif
 	int ds = fr->down_sample;
 	int p8 = 0;
 
+#ifdef USE_3DNOW
+	static func funcs[3][4] =
+	{
+#else
 	static func funcs[2][4] =
 	{
+#endif
 		{mpg123_synth_1to1,
 		 mpg123_synth_2to1,
 		 mpg123_synth_4to1,
@@ -80,6 +110,12 @@ void set_mpg123_synth_functions(struct frame *fr)
 		 mpg123_synth_2to1_8bit,
 		 mpg123_synth_4to1_8bit,
 		 mpg123_synth_ntom_8bit}
+#ifdef USE_3DNOW
+  	       ,{ mpg123_synth_1to1_3dnow,
+  		  mpg123_synth_2to1,
+ 		  mpg123_synth_4to1,
+  		  mpg123_synth_ntom }
+#endif
 	};
 
 	static func_mono funcs_mono[2][2][4] =
@@ -104,11 +140,30 @@ void set_mpg123_synth_functions(struct frame *fr)
 			 mpg123_synth_ntom_8bit_mono}}
 	};
 
+#ifdef USE_3DNOW	
+	static func_dct36 funcs_dct36[2] = {dct36 , dct36_3dnow};
+#endif
+
 	if (mpg123_cfg.resolution == 8)
 		p8 = 1;
 	fr->synth = funcs[p8][ds];
 	fr->synth_mono = funcs_mono[1][p8][ds];
 
+#ifdef USE_3DNOW
+       /* check cpuflags bit 31 (3DNow!) */
+	if( (mpg123_cfg.use_3dnow < 2 ) && 
+	    ((mpg123_cfg.use_3dnow == 1 ) ||
+	     (support_3dnow() == TRUE)) &&
+	    (mpg123_cfg.resolution != 8) )
+	{
+		fr->synth = funcs[2][ds]; /* 3DNow! optimized synth_1to1() */
+		fr->dct36 = funcs_dct36[1]; /* 3DNow! optimized dct36() */
+	}
+	else
+	{
+		fr->dct36 = funcs_dct36[0];
+	}
+#endif
 	if (p8)
 	{
 		mpg123_make_conv16to8_table();
@@ -132,9 +187,14 @@ static void init(void)
 	mpg123_cfg.proxy_use_auth = FALSE;
 	mpg123_cfg.proxy_user = NULL;
 	mpg123_cfg.proxy_pass = NULL;
-	mpg123_cfg.cast_title_streaming = TRUE;
+	mpg123_cfg.cast_title_streaming = FALSE;
+	mpg123_cfg.use_udp_channel = TRUE;
 	mpg123_cfg.use_id3 = TRUE;
+	mpg123_cfg.disable_id3v2 = FALSE;
 	mpg123_cfg.detect_by_content = FALSE;
+#ifdef USE_3DNOW
+	mpg123_cfg.use_3dnow = 0;
+#endif
 
 	filename = g_strconcat(g_get_home_dir(), "/.xmms/config", NULL);
 	if ((cfg = xmms_cfg_open_file(filename)) != 0)
@@ -149,6 +209,7 @@ static void init(void)
 		if (!xmms_cfg_read_string(cfg, "MPG123", "save_http_path", &mpg123_cfg.save_http_path))
 			mpg123_cfg.save_http_path = g_strdup(g_get_home_dir());
 		xmms_cfg_read_boolean(cfg, "MPG123", "cast_title_streaming", &mpg123_cfg.cast_title_streaming);
+		xmms_cfg_read_boolean(cfg, "MPG123", "use_udp_channel", &mpg123_cfg.use_udp_channel);
 
 		xmms_cfg_read_boolean(cfg, "MPG123", "use_proxy", &mpg123_cfg.use_proxy);
 		if (!xmms_cfg_read_string(cfg, "MPG123", "proxy_host", &mpg123_cfg.proxy_host))
@@ -159,10 +220,15 @@ static void init(void)
 		xmms_cfg_read_string(cfg, "MPG123", "proxy_pass", &mpg123_cfg.proxy_pass);
 		
 		xmms_cfg_read_boolean(cfg, "MPG123", "use_id3", &mpg123_cfg.use_id3);
+		xmms_cfg_read_boolean(cfg, "MPG123", "disable_id3v2", &mpg123_cfg.disable_id3v2);
 		if (!xmms_cfg_read_string(cfg, "MPG123", "id3_format", &mpg123_cfg.id3_format))
 			mpg123_cfg.id3_format = g_strdup("%1 - %2");
 		xmms_cfg_read_boolean(cfg, "MPG123", "detect_by_content", &mpg123_cfg.detect_by_content);
 
+#ifdef USE_3DNOW
+		if (!xmms_cfg_read_int(cfg, "MPG123", "use_3dnow", &mpg123_cfg.use_3dnow))
+			mpg123_cfg.use_3dnow=0;
+#endif
 		xmms_cfg_free(cfg);
 	}
 	else
@@ -170,6 +236,9 @@ static void init(void)
 		mpg123_cfg.id3_format = g_strdup("%1 - %2");
 		mpg123_cfg.proxy_host = g_strdup("localhost");
 		mpg123_cfg.save_http_path = g_strdup(g_get_home_dir());
+#ifdef USE_3DNOW
+		mpg123_cfg.use_3dnow = 0;
+#endif
 	}
 }
 
@@ -267,7 +336,9 @@ static guint16 read_wav_id(char *filename)
 					}
 					wavid = buf[0] + 256 * buf[1];
 					seek -= 2;
-					/* we could go on looking for other things, but all we wanted was the wavid */
+					/* we could go on looking for
+                                           other things, but all we
+                                           wanted was the wavid */
 					fclose(file);
 					return wavid;
 				}
@@ -293,14 +364,11 @@ static gboolean mpg123_detect_by_content(gchar *filename)
 	guchar *buf;
 	gint in_buf, i;
 
-#define FAILED() do {fclose(file); return FALSE;} while(0);
-
 	if((file = fopen(filename, "rb")) == NULL)
 		return FALSE;
 	if (fread(tmp, 1, 4, file) != 4)
-		FAILED();
-	if((buf = g_malloc(DET_BUF_SIZE)) == NULL)
-		FAILED();	
+		goto done;
+	buf = g_malloc(DET_BUF_SIZE);
 	head = convert_to_header(tmp);
 	while(!mpg123_head_check(head))
 	{
@@ -313,7 +381,7 @@ static gboolean mpg123_detect_by_content(gchar *filename)
 		if(in_buf == 0)
 		{
 			g_free(buf);
-			FAILED();
+			goto done;
 		}
 		for (i = 0; i < in_buf; i++)
 		{
@@ -335,9 +403,9 @@ static gboolean mpg123_detect_by_content(gchar *filename)
 		 */
 		
 		if (fseek(file, fr.framesize, SEEK_CUR) != 0)
-			FAILED();
+			goto done;
 		if (fread(tmp, 1, 4, file) != 4)
-			FAILED();
+			goto done;
 		head = convert_to_header(tmp);
 		if (mpg123_head_check(head) && mpg123_decode_header(&fr, head))
 		{
@@ -346,6 +414,7 @@ static gboolean mpg123_detect_by_content(gchar *filename)
 		}
 	}
 
+ done:
 	fclose(file);
 	return FALSE;
 }
@@ -384,7 +453,8 @@ static void play_frame(struct frame *fr)
 {
 	if (fr->error_protection)
 	{
-		mpg123_getbits(16);	/* skip crc */
+		bsi.wordpointer += 2;
+		/*  mpg123_getbits(16); */	/* skip crc */
 	}
 	if(!fr->do_layer(fr))
 	{
@@ -412,7 +482,7 @@ enum id3_format_codes
 static const gchar *get_id3_genre(unsigned char genre_code)
 {
 	return (genre_code < GENRE_MAX)
-		? mpg123_id3_genres[genre_code]
+		? _(mpg123_id3_genres[genre_code])
 		: "";
 };
 
@@ -688,27 +758,29 @@ static gchar *get_song_title(FILE * fd, char *filename)
 
 	if (file || (file = fopen(filename, "rb")) != 0)
 	{
-		id3_t *id3;
+		id3_t *id3 = NULL;
 
 		/*
 		 * Try reading ID3v2 tag.
 		 */
-		fseek(file, 0, SEEK_SET);
-		id3 = id3_open_fp(file, 0);
-		if (id3)
+		if (!mpg123_cfg.disable_id3v2)
 		{
-			mpg123_get_id3v2(id3, &id3tag);
-			ret = mpg123_format_song_title(&id3tag, filename);
-			id3_close(id3);
+			fseek(file, 0, SEEK_SET);
+			id3 = id3_open_fp(file, 0);
+			if (id3)
+			{
+				mpg123_get_id3v2(id3, &id3tag);
+				ret = mpg123_format_song_title(&id3tag, filename);
+				id3_close(id3);
+			}
 		}
 
 		/*
 		 * Try reading ID3v1 tag.
 		 */
-		else if ((fseek(file, -1 * sizeof (id3v1tag), SEEK_END) == 0) &&
-			 (fread(&id3v1tag, 1, sizeof (id3v1tag), file)
-			  == sizeof (id3v1tag)) &&
-			 (strncmp(id3v1tag.tag, "TAG", 3) == 0))
+		if (!id3 && (fseek(file, -1 * sizeof (id3v1tag), SEEK_END) == 0) &&
+		    (fread(&id3v1tag, 1, sizeof (id3v1tag), file) == sizeof (id3v1tag)) &&
+		    (strncmp(id3v1tag.tag, "TAG", 3) == 0))
 		{
 			mpg123_id3v1_to_id3v2(&id3v1tag, &id3tag);
 			ret = mpg123_format_song_title(&id3tag, filename);
@@ -737,7 +809,7 @@ static guint get_song_time(FILE * file)
 	struct frame frm;
 	XHEADDATA xing_header;
 	double tpf, bpf;
-	guint32 pos, len;
+	guint32 len;
 
 	if (!file)
 		return -1;
@@ -767,14 +839,13 @@ static guint get_song_time(FILE * file)
 		}
 		g_free(buf);
 		bpf = mpg123_compute_bpf(&frm);
-		pos = ftell(file);
 		fseek(file, 0, SEEK_END);
-		len = ftell(file) - pos + frm.framesize + 4;
+		len = ftell(file);
 		fseek(file, -128, SEEK_END);
 		fread(tmp, 1, 3, file);
 		if (!strncmp(tmp, "TAG", 3))
 			len -= 128;
-		return ((guint) ((len / bpf) * tpf * 1000));
+		return ((guint) ((guint)(len / bpf) * tpf * 1000));
 	}
 	return 0;
 }
@@ -802,13 +873,15 @@ static void get_song_info(char *filename, char **title_real, int *len_real)
 
 static void *decode_loop(void *arg)
 {
-	gboolean read_ok = 0;
-	gint disp_count = 0,temp_time;
+	gboolean output_opened = FALSE;
+	gint disp_count = 0, temp_time;
 	gchar *filename = arg;
 
+	/* This is used by fileinfo on http streams */
+	mpg123_bitrate = 0;
+
 	mpg123_pcm_sample = (unsigned char *) g_malloc0(32768);
 	mpg123_pcm_point = 0;
-	mpg123_info->jump_to_time = -1;
 	mpg123_filename = filename;
 
 
@@ -867,6 +940,8 @@ static void *decode_loop(void *arg)
 		mpg123_stereo = fr.stereo;
 		mpg123_layer = fr.lay;
 		mpg123_lsf = fr.lsf;
+		mpg123_mpeg25 = fr.mpeg25;
+		mpg123_mode = fr.mode;
 
 		if (strncasecmp(filename, "http://", 7))
 		{
@@ -881,6 +956,7 @@ static void *decode_loop(void *arg)
 			mpg123_length = -1;
 		}
 		mpg123_ip.set_info(mpg123_title, mpg123_length, mpg123_bitrate * 1000, mpg123_freqs[fr.sampling_frequency], fr.stereo);
+		output_opened = TRUE;
 		if (!mpg123_ip.output->open_audio(mpg123_cfg.resolution == 16 ? FMT_S16_NE : FMT_U8,
 		       mpg123_freqs[fr.sampling_frequency] >> mpg123_cfg.downsample,
 					  mpg123_cfg.channels == 2 ? fr.stereo : 1))
@@ -908,7 +984,7 @@ static void *decode_loop(void *arg)
 		}
 		if (!mpg123_info->eof)
 		{
-			if ((read_ok = mpg123_read_frame(&fr)) != 0)
+			if (mpg123_read_frame(&fr) != 0)
 			{
 				if(fr.lay != mpg123_layer || fr.lsf != mpg123_lsf)
 				{
@@ -968,8 +1044,7 @@ static void *decode_loop(void *arg)
 							mpg123_info->output_audio = FALSE;
 							continue;
 						}
-					}
-					
+					}					
 				}
 				
 				if (tabsel_123[fr.lsf][fr.lay - 1][fr.bitrate_index] != mpg123_bitrate)
@@ -987,7 +1062,6 @@ static void *decode_loop(void *arg)
 							mpg123_info->tpf = mpg123_compute_tpf(&fr);
 							mpg123_length = mpg123_info->num_frames * mpg123_info->tpf * 1000;
 						}
-							
 						mpg123_ip.set_info(mpg123_title, mpg123_length, mpg123_bitrate * 1000, mpg123_frequency, mpg123_stereo);
 					}
 				}
@@ -1008,11 +1082,11 @@ static void *decode_loop(void *arg)
 			xmms_usleep(10000);
 		}
 	}
-	mpg123_filename = NULL;
 	g_free(mpg123_title);
 	mpg123_title = NULL;
 	mpg123_stream_close();
-	mpg123_ip.output->close_audio();
+	if (output_opened && !audio_error)
+		mpg123_ip.output->close_audio();
 	g_free(mpg123_pcm_sample);
 	pthread_exit(NULL);
 }
@@ -1026,6 +1100,7 @@ static void play_file(char *filename)
 	mpg123_info->going = 1;
 	mpg123_info->first_frame = TRUE;
 	mpg123_info->output_audio = TRUE;
+	mpg123_info->jump_to_time = -1;
 	skip_frames = 0;
 	audio_error = FALSE;
 	have_xing_header = FALSE;
@@ -1035,15 +1110,13 @@ static void play_file(char *filename)
 
 static void stop(void)
 {
-	if (mpg123_info)
+	if (mpg123_info && mpg123_info->going)
 	{
-		if (mpg123_info->going)
-		{
-			mpg123_info->going = FALSE;
-			pthread_join(decode_thread, NULL);
-			g_free(mpg123_info);
-			mpg123_info = NULL;
-		}
+		mpg123_filename = NULL;
+		mpg123_info->going = FALSE;
+		pthread_join(decode_thread, NULL);
+		g_free(mpg123_info);
+		mpg123_info = NULL;
 	}
 }
 
@@ -1075,9 +1148,9 @@ InputPlugin mpg123_ip =
 {
 	NULL,
 	NULL,
-	"MPEG Layer 1/2/3 Player " VERSION,
+	NULL, /* Description */
 	init,
-	NULL,
+	aboutbox,
 	mpg123_configure,
 	is_our_file,
 	NULL,
@@ -1096,5 +1169,70 @@ InputPlugin mpg123_ip =
 
 InputPlugin *get_iplugin_info(void)
 {
+	mpg123_ip.description =
+		g_strdup_printf(_("MPEG Layer 1/2/3 Player %s"), VERSION);
 	return &mpg123_ip;
 }
+
+#ifdef USE_3DNOW
+static void aboutbox()
+{
+	GtkWidget *dialog_vbox1;
+	GtkWidget *hbox1;
+	GtkWidget *label1;
+	GtkWidget *dialog_action_area1;
+	GtkWidget *about_exit;
+
+	if (!about_window)
+	{
+		about_window = gtk_dialog_new();
+		gtk_object_set_data(GTK_OBJECT(about_window), "about_window", about_window);
+		gtk_window_set_title(GTK_WINDOW(about_window), "About MPEG Layer 1/2/3 plugin (support 3DNow!)");
+		gtk_window_set_policy(GTK_WINDOW(about_window), FALSE, FALSE, FALSE);
+		gtk_window_set_position(GTK_WINDOW(about_window), GTK_WIN_POS_MOUSE);
+		gtk_signal_connect(GTK_OBJECT(about_window), "destroy", GTK_SIGNAL_FUNC(gtk_widget_destroyed), &about_window);
+		gtk_container_border_width(GTK_CONTAINER(about_window), 10);
+
+		dialog_vbox1 = GTK_DIALOG(about_window)->vbox;
+		gtk_object_set_data(GTK_OBJECT(about_window), "dialog_vbox1", dialog_vbox1);
+		gtk_widget_show(dialog_vbox1);
+		gtk_container_border_width(GTK_CONTAINER(dialog_vbox1), 5);
+
+		hbox1 = gtk_hbox_new(FALSE, 0);
+		gtk_object_set_data(GTK_OBJECT(about_window), "hbox1", hbox1);
+		gtk_widget_show(hbox1);
+		gtk_box_pack_start(GTK_BOX(dialog_vbox1), hbox1, TRUE, TRUE, 0);
+		gtk_container_border_width(GTK_CONTAINER(hbox1), 5);
+		gtk_widget_realize(about_window);
+
+		label1 = gtk_label_new("MPEG Layer 1/2/3 Plugin\n3DNow! enhancement patch ported to xmms by O.Kayasono\nhttp://village.infoweb.ne.jp/~kayasono/linux/xmms.htm\n\nOriginal 3DNow! improvement patch for mpg123 by Takuhiro KIMURA\nhttp://hannah.ipc.miyakyo-u.ac.jp/kim/Linux/mpg123-3dn.e.html\n");
+		gtk_object_set_data(GTK_OBJECT(about_window), "label1", label1);
+		gtk_widget_show(label1);
+		gtk_box_pack_start(GTK_BOX(hbox1), label1, TRUE, TRUE, 0);
+
+		dialog_action_area1 = GTK_DIALOG(about_window)->action_area;
+		gtk_object_set_data(GTK_OBJECT(about_window), "dialog_action_area1", dialog_action_area1);
+		gtk_widget_show(dialog_action_area1);
+		gtk_container_border_width(GTK_CONTAINER(dialog_action_area1), 10);
+
+		about_exit = gtk_button_new_with_label("Ok");
+		gtk_signal_connect_object(GTK_OBJECT(about_exit), "clicked",
+					  GTK_SIGNAL_FUNC
+					  (gtk_widget_destroy),
+					  GTK_OBJECT(about_window));
+
+		gtk_object_set_data(GTK_OBJECT(about_window), "about_exit", about_exit);
+		gtk_widget_show(about_exit);
+		gtk_box_pack_start(GTK_BOX(dialog_action_area1), about_exit, TRUE, TRUE, 0);
+
+		gtk_widget_show(about_window);
+	}
+	else
+	{
+		gdk_window_raise(about_window->window);
+	}
+}
+#endif
+
+
+
