@@ -1,5 +1,5 @@
 /*  XMMS - Cross-platform multimedia player
- *  Copyright (C) 1998-1999  Peter Alm, Mikael Alm, Olle Hallnas, Thomas Nilsson and 4Front Technologies
+ *  Copyright (C) 1998-2000  Peter Alm, Mikael Alm, Olle Hallnas, Thomas Nilsson and 4Front Technologies
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -23,7 +23,9 @@
 #include <stdlib.h>
 #include <stdio.h>
 
+#include "xmms/i18n.h"
 #include "config.h"
+#include <locale.h>
 
 #include "libxmms/xmmsctrl.h"
 #include "xmms-dock-master.xpm"
@@ -581,7 +583,7 @@ void init(void)
 	pixmap = gdk_pixmap_create_from_xpm(window->window, &mask, NULL, icon_name);
 	if (!pixmap)
 	{
-		printf("ERROR: Couldn't find %s\n", icon_name);
+		printf(_("ERROR: Couldn't find %s\n"), icon_name);
 		g_free(icon_name);
 		gtk_exit(1);
 	}
@@ -615,17 +617,19 @@ void init(void)
 
 void display_usage(char *cmd)
 {
-	printf("Usage: %s [options]\n\n\
-Options:\n\
---------\n\n\
--h, --help		Display this text and exit.\n\
--g, --geometry		Set the geometry (for example +20+20)\n\
--s, --session		Set the xmms session to use (Default: 0)\n\
--c, --command		Command to launch xmms (Default: xmms)\n\
--i, --icon		Set the icon to use when xmms is not running\n\
--n, --single		Only a single click is needed to start xmms\n\
--t, --title		Display song title when mouse is in window\n\
--v, --version		Display version information and exit\n\n", cmd);
+	printf(_(
+		"Usage: %s [options]\n\n"
+		"Options:\n"
+		"--------\n\n"
+		"-h, --help		Display this text and exit.\n"
+		"-g, --geometry		Set the geometry (for example +20+20)\n"
+		"-s, --session		Set the xmms session to use (Default: 0)\n"
+		"-c, --command		Command to launch xmms (Default: xmms)\n"
+		"-i, --icon		Set the icon to use when xmms is not running\n"
+		"-n, --single		Only a single click is needed to start xmms\n"
+		"-t, --title		Display song title when mouse is in window\n"
+		"-v, --version		Display version information and exit\n\n"),
+	       cmd);
 }
 
 struct option long_options[] =
@@ -643,9 +647,14 @@ struct option long_options[] =
 
 int main(int argc, char **argv)
 {
-
 	gint c, o;
 
+	gtk_set_locale();
+#ifdef ENABLE_NLS
+	bindtextdomain (PACKAGE, LOCALEDIR);
+	textdomain (PACKAGE);
+#endif
+
 	gtk_init(&argc, &argv);
 
 	while ((c = getopt_long(argc, argv, "hg:s:c:i:ntv", long_options, &o)) != -1)
