@@ -1,5 +1,5 @@
 /*  XMMS - Cross-platform multimedia player
- *  Copyright (C) 1998-1999  Peter Alm, Mikael Alm, Olle Hallnas, Thomas Nilsson and 4Front Technologies
+ *  Copyright (C) 1998-2000  Peter Alm, Mikael Alm, Olle Hallnas, Thomas Nilsson and 4Front Technologies
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -46,7 +46,7 @@ void playlist_clear(void)
 	PlaylistEntry *entry;
 	
 	if (get_input_playing())
-	     input_stop();
+		input_stop();
 
 	
 	pthread_mutex_lock(&playlist_mutex);
@@ -73,6 +73,75 @@ void playlist_clear(void)
 	playlist_generate_shuffle_list();
 }
 
+void playlist_delete_index(glong index)
+{
+	gboolean restart_playing = FALSE, set_info_text = FALSE;
+	GList *node, *playing_song;
+	PlaylistEntry *entry;
+	
+	PL_LOCK();
+	if (!playlist)
+	{
+		PL_UNLOCK();
+		return;
+	}
+
+	node = g_list_nth(playlist, index);
+	if(!node)
+	{
+		PL_UNLOCK();
+		return;
+	}
+
+	/*
+	 * We call g_list_find manually here because
+	 * we don't want an item in the shuffle_list
+	 */
+	playing_song = g_list_find(playlist, playlist_position);
+	entry = (PlaylistEntry *) node->data;
+	if (playing_song == node)
+	{
+		set_info_text = TRUE;
+		if (get_input_playing())
+		{
+			PL_UNLOCK();
+			input_stop();
+			PL_LOCK();
+			restart_playing = TRUE;
+		}
+		if (g_list_next(playing_song))
+			playlist_position = g_list_next(playing_song)->data;
+		else if (g_list_previous(playing_song))
+			playlist_position = g_list_previous(playing_song)->data;
+		else if (node != playlist)
+			playlist_position = playlist->data;
+		else
+			playlist_position = NULL;
+	}
+	if (entry->filename)
+		g_free(entry->filename);
+	if (entry->title)
+		g_free(entry->title);
+	playlist = g_list_remove_link(playlist, node);
+	g_free(entry);
+	g_list_free_1(node);
+
+	PL_UNLOCK();
+
+	playlistwin_update_list();
+	playlist_generate_shuffle_list();
+	if (restart_playing)
+	{
+		if (playlist_position)
+			playlist_play();
+		else
+			mainwin_set_song_info(0, 0, 0);
+	}
+	else if (set_info_text)
+		mainwin_set_info_text();
+
+}
+
 void playlist_delete(gboolean crop)
 {
 	gboolean restart_playing = FALSE, set_info_text = FALSE, regen_shuffle = FALSE;
@@ -92,7 +161,8 @@ void playlist_delete(gboolean crop)
 		{
 			regen_shuffle = TRUE;
 			/*
-			 * We call g_list_find manually here because we don't want an item in the shuffle_list
+			 * We call g_list_find manually here because we don't
+			 * want an item in the shuffle_list
 			 */
 			plist_pos_list = g_list_find(playlist, playlist_position);
 			if (plist_pos_list == node)
@@ -210,37 +280,36 @@ void playlist_ins(gchar * filename, glong pos)
 		playlist_load_ins(filename, pos);
 	else
 		if (input_check_file(filename))
+		{
 			__playlist_ins(filename, pos);
+			playlist_generate_shuffle_list();
+		}
 }
 
-/*  void playlist_add_dir(gchar * path) */
-/*  { */
-/*  	playlist_ins_dir(path, -1); */
-/*  } */
-
-guint playlist_ins_dir(gchar * path, glong pos)
+static guint playlist_ins_dir_real(gchar * path, glong pos)
 {
 	DIR *dir;
 	struct dirent *dirent;
 	struct stat statbuf;
-	gchar *filename, *temp;
+	gchar *temp;
 	guint entries = 0;
-	GList *list, *node;
+	GList *list;
 
 	if (path[strlen(path) - 1] != '/')
-		temp = g_strdup_printf("%s/", path);
+		temp = g_strconcat(path, "/", NULL);
 	else
 		temp = g_strdup(path);
 
 	if ((list = input_scan_dir(temp)) != NULL)
 	{
+		GList *node;
 		g_free(temp);
 		node = list;
 		while (node)
 		{
 			__playlist_ins(node->data, pos);
 			entries++;
-			if (pos > 0)
+			if (pos >= 0)
 				pos++;
 			g_free(node->data);
 			node = g_list_next(node);
@@ -249,47 +318,48 @@ guint playlist_ins_dir(gchar * path, glong pos)
 		playlist_generate_shuffle_list();
 		return entries;
 	}
-	g_free(temp);
+
 	if ((dir = opendir(path)) != NULL)
 	{
 		while ((dirent = readdir(dir)) != NULL)
 		{
 			if (dirent->d_name[0] != '.')
 			{
-				filename = g_strconcat(path, "/", dirent->d_name, NULL);
+				gchar *filename = g_strconcat(temp, dirent->d_name, NULL);
+				/* Why don't we follow symlinks? */
 				lstat(filename, &statbuf);
 				if (S_ISDIR(statbuf.st_mode))
 				{
 					guint i;
-					i = playlist_ins_dir(filename, pos);
+					i = playlist_ins_dir_real(filename, pos);
 					entries += i;
-					if (pos > 0)
+					if (pos >= 0)
 						pos += i;
 				}
 				else if (input_check_file(filename))
 				{
 					__playlist_ins(filename, pos);
 					entries++;
-					if (pos > 0)
+					if (pos >= 0)
 						pos++;
 				}
 				g_free(filename);
 			}
 		}
-		playlist_generate_shuffle_list();
 		closedir(dir);
 	}
+	g_free(temp);
 	return entries;
 }
 
-/*
- * I'm not sure if this is the correct way to do it but it seems to work fine
- */
+guint playlist_ins_dir(gchar * path, glong pos)
+{
+	guint entries;
 
-/*  void playlist_add_url_string(gchar * string) */
-/*  { */
-/*  	playlist_ins_url_string(string, -1); */
-/*  } */
+	entries = playlist_ins_dir_real(path, pos);
+	playlist_generate_shuffle_list();
+	return entries;
+}
 
 guint playlist_ins_url_string(gchar * string, glong pos)
 {
@@ -306,8 +376,10 @@ guint playlist_ins_url_string(gchar * string, glong pos)
 				*(temp - 1) = '\0';
 			*temp = '\0';
 		}
-		if (!strncasecmp(string, "file:", 5))
-			string = string + 5;
+		if (!strncasecmp(string, "file:", 5)) {
+			if(!xmms_urldecode_path(string))
+				return 0;
+		}
 
 		stat(string, &statbuf);
 		if (S_ISDIR(statbuf.st_mode))
@@ -324,7 +396,7 @@ guint playlist_ins_url_string(gchar * string, glong pos)
 			}
 		}
 		entries += i;
-		if (pos > 0)
+		if (pos >= 0)
 			pos += i;
 		if (!temp)
 			break;
@@ -369,7 +441,7 @@ void playlist_play(void)
 	svis_clear_data(mainwin_svis);
 
 	PL_LOCK();
-	entry = g_list_nth(playlist, get_playlist_position())->data;
+	entry = g_list_nth(playlist, __get_playlist_position())->data;
 	PL_UNLOCK();
 	
 	input_play(entry->filename);
@@ -388,16 +460,14 @@ void playlist_set_info(gchar * title, gint length, gint rate, gint freq, gint nc
 	PlaylistEntry *entry;
 
 	PL_LOCK();
-	entry = g_list_nth(playlist, get_playlist_position())->data;
+	entry = g_list_nth(playlist, __get_playlist_position())->data;
 
-	if (entry && entry->title)
+	if (entry)
 	{
 		g_free(entry->title);
-		entry->title = NULL;
-	}
-	if (entry && title)
 		entry->title = g_strdup(title);
-	entry->length = length;
+		entry->length = length;
+	}
 	PL_UNLOCK();
 	mainwin_set_song_info(rate, freq, nch);
 	mainwin_set_info_text();
@@ -424,6 +494,7 @@ void playlist_check_pos_current(void)
 void playlist_next(void)
 {
 	GList *plist_pos_list;
+	gboolean restart_playing = FALSE;
 
 	PL_LOCK();
 	if (!playlist)
@@ -439,6 +510,15 @@ void playlist_next(void)
 		PL_UNLOCK();
 		return;
 	}
+	
+	if (get_input_playing())
+	{
+		/* We need to stop before changing playlist_position */
+		PL_UNLOCK();
+		input_stop();
+		PL_LOCK();
+		restart_playing = TRUE;
+	}
 
 	if (g_list_next(plist_pos_list))
 		playlist_position = g_list_next(plist_pos_list)->data;
@@ -455,11 +535,9 @@ void playlist_next(void)
 	}
 	PL_UNLOCK();
 	playlist_check_pos_current();
-	if (get_input_playing())
-	{
-		input_stop();
+
+	if (restart_playing)
 		playlist_play();
-	}
 	else
 	{
 		mainwin_set_info_text();
@@ -470,6 +548,7 @@ void playlist_next(void)
 void playlist_prev(void)
 {
 	GList *plist_pos_list;
+	gboolean restart_playing = FALSE;
 	
 	PL_LOCK();
 	if (!playlist)
@@ -486,16 +565,22 @@ void playlist_prev(void)
 		return;
 	}
 
+	if (get_input_playing())
+	{
+		/* We need to stop before changing playlist_position */
+		PL_UNLOCK();
+		input_stop();
+		PL_LOCK();
+		restart_playing = TRUE;
+	}
 	
 	if (g_list_previous(plist_pos_list))
 		playlist_position = g_list_previous(plist_pos_list)->data;
 	PL_UNLOCK();
 	playlist_check_pos_current();
-	if (get_input_playing())
-	{
-		input_stop();
+
+	if (restart_playing)
 		playlist_play();
-	}
 	else
 	{
 		mainwin_set_info_text();
@@ -506,6 +591,8 @@ void playlist_prev(void)
 void playlist_set_position(gint pos)
 {
 	GList *node;
+	gboolean restart_playing = FALSE;
+	
 	PL_LOCK();
 	if (!playlist)
 	{
@@ -520,15 +607,21 @@ void playlist_set_position(gint pos)
 		return;
 	}
 	
-	playlist_position = node->data;
-	PL_UNLOCK();
-	playlist_generate_shuffle_list();
-	playlist_check_pos_current();
 	if (get_input_playing())
 	{
+		/* We need to stop before changing playlist_position */
+		PL_UNLOCK();
 		input_stop();
-		playlist_play();
+		PL_LOCK();
+		restart_playing = TRUE;
 	}
+
+	playlist_position = node->data;
+	PL_UNLOCK();
+	playlist_check_pos_current();
+
+	if (restart_playing)
+		playlist_play();
 	else
 	{
 		mainwin_set_info_text();
@@ -604,41 +697,42 @@ gint __get_playlist_length(void)
 gchar *playlist_get_info_text(void)
 {
 	PlaylistEntry *entry;
-	gchar *text = NULL, *title, *tmp, *tmp2;
+	gchar *text, *title, *tmp, *tmp2;
 
 	PL_LOCK();
-	if (playlist)
+	if (!playlist)
 	{
-		entry = (PlaylistEntry *) g_list_nth(playlist, get_playlist_position())->data;
-		if (entry->title)
-			title = entry->title;
-		else
-		{
-			title = strrchr(entry->filename, '/');
-			if (!title)
-				title = entry->filename;
-			else
-				title++;
-		}
-		text = g_malloc(strlen(title) + 20);
-		if (entry->length != -1)
-			sprintf(text, "%d. %s (%d:%-2.2d)", get_playlist_position() + 1, title, entry->length / 60000, (entry->length / 1000) % 60);
-		else
-			sprintf(text, "%d. %s", get_playlist_position() + 1, title);
-		if (cfg.convert_underscore)
-			while ((tmp = strchr(text, '_')) != NULL)
-				*tmp = ' ';
-		if (cfg.convert_twenty)
-			while ((tmp = strstr(text, "%20")) != NULL)
-			{
-				tmp2 = tmp + 3;
-				*(tmp++) = ' ';
-				while (*tmp2)
-					*(tmp++) = *(tmp2++);
-				*tmp = '\0';
-			}
-
+		PL_UNLOCK();
+		return NULL;
 	}
+
+	entry = (PlaylistEntry *) g_list_nth(playlist, __get_playlist_position())->data;
+	if (entry->title)
+		title = entry->title;
+	else
+		title = g_basename(entry->filename);
+	
+	text = g_malloc(strlen(title) + 20);
+	if (entry->length != -1)
+		text = g_strdup_printf("%d. %s (%d:%-2.2d)",
+				       __get_playlist_position() + 1,
+				       title, entry->length / 60000,
+				       (entry->length / 1000) % 60);
+	else
+		text = g_strdup_printf("%d. %s", __get_playlist_position() + 1, title);
+	if (cfg.convert_underscore)
+		while ((tmp = strchr(text, '_')) != NULL)
+			*tmp = ' ';
+	if (cfg.convert_twenty)
+		while ((tmp = strstr(text, "%20")) != NULL)
+		{
+			tmp2 = tmp + 3;
+			*(tmp++) = ' ';
+			while (*tmp2)
+				*(tmp++) = *(tmp2++);
+			*tmp = '\0';
+		}
+	
 	PL_UNLOCK();
 	return text;
 }
@@ -648,7 +742,7 @@ int playlist_get_current_length(void)
 	gint retval = 0;
 
 	PL_LOCK();
-	if (playlist)
+	if (playlist && playlist_position)
 		retval = playlist_position->length;
 	PL_UNLOCK();
 	return retval;
@@ -754,7 +848,7 @@ guint playlist_load_ins(gchar * filename, glong pos)
 			{
 				playlist_load_ins_file(line, filename, pos);
 				entries ++;
-				if (pos > 0)
+				if (pos >= 0)
 					pos++;
 				g_free(line);
 			}
@@ -777,9 +871,11 @@ guint playlist_load_ins(gchar * filename, glong pos)
 				}
 				while (line[strlen(line) - 1] == '\r' || line[strlen(line) - 1] == '\n')
 					line[strlen(line) - 1] = '\0';
+				if (line[0] == '#')
+					continue;
 				playlist_load_ins_file(line, filename, pos);
 				entries++;
-				if (pos > 0)
+				if (pos >= 0)
 					pos++;
 			}
 			fclose(file);
@@ -787,15 +883,16 @@ guint playlist_load_ins(gchar * filename, glong pos)
 			return entries;
 		}
 	}
-	return FALSE;
+	return 0;
 }
 
 GList *get_playlist(void)
 {
+	/* Caller should hold playlist_mutex */
 	return playlist;
 }
 
-gint get_playlist_position(void)
+gint __get_playlist_position(void)
 {
 	/* Caller should hold playlist_mutex */
 	if (playlist && playlist_position)
@@ -803,6 +900,129 @@ gint get_playlist_position(void)
 	return 0;
 }
 
+gint get_playlist_position(void)
+{
+	gint retval;
+
+	PL_LOCK();
+	retval = __get_playlist_position();
+	PL_UNLOCK();
+
+	return retval;
+}
+
+gchar * playlist_get_filename(gint pos)
+{
+	gchar *ret;
+	PlaylistEntry *entry;
+	GList *node;
+	
+	PL_LOCK();
+	if (!playlist)
+	{
+		PL_UNLOCK();
+		return NULL;
+	}
+	node = g_list_nth(playlist, pos);
+	if (!node)
+	{
+		PL_UNLOCK();
+		return NULL;
+	}
+	entry = node->data;
+	
+	ret = g_strdup(entry->filename);
+	PL_UNLOCK();
+
+	return ret;
+}
+
+gchar * playlist_get_songtitle(gint pos)
+{
+	gchar *title, *filename;
+	PlaylistEntry *entry;
+	GList *node;
+
+	PL_LOCK();
+	if (!playlist)
+	{
+		PL_UNLOCK();
+		return NULL;
+	}
+	node = g_list_nth(playlist, pos);
+	if (!node)
+	{
+		PL_UNLOCK();
+		return NULL;
+	}
+	entry = node->data;
+
+	filename = g_strdup(entry->filename);
+	
+	if (entry->title == NULL && entry->length == -1)
+	{
+		gint dummy;
+
+		PL_UNLOCK();
+		input_get_song_info(filename, &title, &dummy);
+	}
+	else
+	{
+		title = g_strdup(entry->title);
+		PL_UNLOCK();
+	}
+
+	if (title != NULL)
+	{
+		g_free(filename);
+		return title;
+	}
+
+	title = g_strdup(g_basename(filename));
+	g_free(filename);
+
+	return title;
+}
+
+gint playlist_get_songtime(gint pos)
+{
+	gint retval;
+	PlaylistEntry *entry;
+	GList *node;
+	
+	PL_LOCK();
+	if (!playlist)
+	{
+		PL_UNLOCK();
+		return -1;
+	}
+	node = g_list_nth(playlist, pos);
+	if (!node)
+	{
+		PL_UNLOCK();
+		return -1;
+	}
+	entry = node->data;
+
+	if (entry->title == NULL && entry->length == -1)
+	{
+		gchar *dummy, *filename;
+
+		filename = g_strdup(entry->filename);
+		PL_UNLOCK();
+		input_get_song_info(filename, &dummy, &retval);
+		g_free(filename);
+		g_free(dummy);
+	}
+	else
+	{
+		retval = entry->length;
+		PL_UNLOCK();
+	}
+
+	return retval;
+}
+
 gint playlist_sort_by_title_cmpfunc(PlaylistEntry * a, PlaylistEntry * b)
 {
 	gchar *a_title = NULL, *b_title = NULL;
@@ -906,7 +1126,7 @@ static GList* playlist_sort_selected(GList *list, GCompareFunc cmpfunc)
 	{
 		if(!list1)
 		{
-			g_log(NULL, G_LOG_LEVEL_CRITICAL, "%s: Error during list sorting\nPossibly dropped some playlist-entries.", PACKAGE);
+			g_log(NULL, G_LOG_LEVEL_CRITICAL, "%s: Error during list sorting. Possibly dropped some playlist-entries.", PACKAGE);
 			break;
 		}
 		list = g_list_insert(list, list1->data, GPOINTER_TO_INT(list2->data));
@@ -948,35 +1168,54 @@ void playlist_reverse(void)
 	PL_UNLOCK();
 }
 
+static GList *playlist_shuffle_list(GList *list)
+{
+	gint len = g_list_length(list);
+	gint i, j;
+	GList *node, **ptrs;
+
+	if (!len)
+		return 0;
+
+	ptrs = g_new(GList *, len);
+
+	for (node = list, i = 0; i < len; node = g_list_next(node), i++)
+		ptrs[i] = node;
+
+	list = 0;
+
+	j = rand() % len;
+	list = ptrs[j];
+	ptrs[j]->next = 0;
+	ptrs[j] = ptrs[0];
+
+	for (i = 1; i < len; i++)
+	{
+		j = rand() % (len - i);
+		list->prev = ptrs[i + j];
+		ptrs[i + j]->next = list;
+		list = ptrs[i + j];
+		ptrs[i + j] = ptrs[i];
+	}
+	list->prev = 0;
+
+	g_free(ptrs);
+
+	return list;
+}
+
 void playlist_random(void)
 {
-	GList *new_list, *node, *prev;
-	gint numsongs;
-
 	PL_LOCK();
 
-	numsongs = g_list_length(playlist);
+	playlist = playlist_shuffle_list(playlist);
 
-	new_list = g_list_nth(playlist, (gint) ((gfloat) numsongs-- * rand() / (RAND_MAX + 1.0)));
-	playlist = g_list_remove_link(playlist, new_list);
-	prev = new_list;
-
-	while (numsongs)
-	{
-		node = g_list_nth(playlist, (gint) ((gfloat) numsongs-- * rand() / (RAND_MAX + 1.0)));
-		playlist = g_list_remove_link(playlist, node);
-		prev->next = node;
-		node->prev = prev;
-		prev = node;
-	}
-
-	playlist = new_list;
 	PL_UNLOCK();
 }
 
 void playlist_generate_shuffle_list(void)
 {
-	GList *new_list = NULL, *node;
+	GList *node;
 	gint numsongs;
 	gboolean update_mainwin = FALSE;
 
@@ -989,12 +1228,9 @@ void playlist_generate_shuffle_list(void)
 	}
 	if (playlist)
 	{
-		node = playlist;
-		while (node)
-		{
+		for (node = playlist; node; node = node->next)
 			shuffle_list = g_list_append(shuffle_list, node->data);
-			node = g_list_next(node);
-		}
+		shuffle_list = playlist_shuffle_list(shuffle_list);
 		numsongs = g_list_length(shuffle_list);
 
 		if (!numsongs)
@@ -1006,24 +1242,10 @@ void playlist_generate_shuffle_list(void)
 		if(playlist_position)
 		{
 			node = g_list_nth(shuffle_list, g_list_index(shuffle_list, playlist_position));
-			numsongs--;
+			shuffle_list = g_list_remove_link(shuffle_list, node);
+			shuffle_list = g_list_prepend(shuffle_list, node->data);
 		}
 		else
-			node = g_list_nth(shuffle_list, (gint) ((gfloat) numsongs-- * rand() / (RAND_MAX + 1.0)));
-		new_list = g_list_append(new_list, node->data);
-		shuffle_list = g_list_remove_link(shuffle_list, node);
-		g_list_free_1(node);
-		
-		while (numsongs)
-		{
-			node = g_list_nth(shuffle_list, (gint) ((gfloat) numsongs-- * rand() / (RAND_MAX + 1.0)));
-			shuffle_list = g_list_remove_link(shuffle_list, node);
-			new_list = g_list_append(new_list, node->data);
-			g_list_free_1(node);
-		}
-		shuffle_list = new_list;
-
-		if (!playlist_position)
 		{
 			if (cfg.shuffle)
 				playlist_position = shuffle_list->data;
@@ -1056,7 +1278,7 @@ void playlist_fileinfo_current(void)
 	PL_LOCK();
 	
 	if (get_playlist())
-		path = g_strdup(((PlaylistEntry *) g_list_nth(get_playlist(), get_playlist_position())->data)->filename);
+		path = g_strdup(((PlaylistEntry *) g_list_nth(get_playlist(), __get_playlist_position())->data)->filename);
 	PL_UNLOCK();
 	if (path)
 	{
@@ -1075,9 +1297,9 @@ void *playlist_get_info_func(void *arg)
 
 	while (playlist_get_info_going)
 	{
-		pthread_mutex_lock(&playlist_mutex);
 		if (cfg.get_info_on_load && playlist_get_info_scan_active)
 		{
+			PL_LOCK();
 			for (node = get_playlist(); node; node = g_list_next(node))
 			{
 				entry = node->data;
@@ -1104,7 +1326,7 @@ void *playlist_get_info_func(void *arg)
 					if (playlistwin_item_visible(g_list_position(get_playlist(), node)))
 						update_playlistwin = TRUE;
 					if (entry == (PlaylistEntry *) g_list_nth(get_playlist(),
-										  get_playlist_position())->data)
+										  __get_playlist_position())->data)
 						update_mainwin = TRUE;
 					break;
 				}
@@ -1115,22 +1337,36 @@ void *playlist_get_info_func(void *arg)
 					continue;
 				}	
 			}
+			PL_UNLOCK();
 
 			if (!node)
 				playlist_get_info_scan_active = FALSE;
 		}
-		else if (cfg.get_info_on_demand && cfg.playlist_visible && !cfg.playlist_shaded && playlist)
+		else if (!cfg.get_info_on_load && cfg.get_info_on_demand &&
+			 cfg.playlist_visible && !cfg.playlist_shaded)
 		{
-			for (node = g_list_nth(get_playlist(), playlistwin_get_toprow()); node && playlistwin_item_visible(g_list_position(get_playlist(), node)); node = g_list_next(node))
+			gboolean found = FALSE;
+
+			PL_LOCK();
+			if (!get_playlist())
+			{
+				PL_UNLOCK();
+				xmms_usleep(1000000);
+				continue;
+			}
+			
+			for (node = g_list_nth(get_playlist(), playlistwin_get_toprow());
+			     node && playlistwin_item_visible(g_list_position(get_playlist(), node));
+			     node = g_list_next(node))
 			{
 				entry = node->data;
 				if (entry->title || entry->length != -1)
 					continue;
-				
+
 				temp_filename = g_strdup(entry->filename);
 				temp_title = NULL;
 				temp_length = -1;
-
+				
 				/* We don't want to lock the playlist while reading info */
 				PL_UNLOCK();
 				input_get_song_info(temp_filename, &temp_title, &temp_length);
@@ -1142,12 +1378,13 @@ void *playlist_get_info_func(void *arg)
 					/* entry is still around */
 					if (!temp_title && temp_length == -1)
 						continue;
+					found = TRUE;
 					entry->title = temp_title;
 					entry->length = temp_length;
 					if (playlistwin_item_visible(g_list_position(get_playlist(), node)))
 						update_playlistwin = TRUE;
 					if (entry == (PlaylistEntry *) g_list_nth(get_playlist(),
-										  get_playlist_position())->data)
+										  __get_playlist_position())->data)
 						update_mainwin = TRUE;
 					break;
 				}
@@ -1156,10 +1393,18 @@ void *playlist_get_info_func(void *arg)
 					/* entry wasn't around anymore so we restart. */
 					node = g_list_nth(get_playlist(), playlistwin_get_toprow());
 					continue;
-				}				
+				}
+			}
+			PL_UNLOCK();
+			if (!found)
+			{
+				xmms_usleep(500000);
+				continue;
 			}
 		}
-		pthread_mutex_unlock(&playlist_mutex);
+		else
+			xmms_usleep(500000);
+
 		if (update_playlistwin)
 		{
 			playlistwin_update_list();
@@ -1170,7 +1415,6 @@ void *playlist_get_info_func(void *arg)
 			mainwin_set_info_text();
 			update_mainwin = FALSE;
 		}
-		xmms_usleep(10000);
 	}
 	pthread_exit(NULL);
 }
@@ -1180,6 +1424,7 @@ void playlist_remove_dead_files(void)
 	/* FIXME? Does virtual directories work well? */
 	GList *node, *next_node, *curr_pos, *temp = NULL;
 	PlaylistEntry *entry;
+	gboolean list_changed = FALSE;
 	
 	PL_LOCK();
 	node = playlist;
@@ -1194,6 +1439,7 @@ void playlist_remove_dead_files(void)
 		    ((temp = input_scan_dir(entry->filename)) == NULL) &&
 		    access(entry->filename, R_OK))
 		{
+			list_changed = TRUE;
 			curr_pos = g_list_find(playlist, playlist_position);
 			if (node == curr_pos)
 			{
@@ -1227,7 +1473,12 @@ void playlist_remove_dead_files(void)
 		node = next_node;
 	}
 	PL_UNLOCK();
-	playlistwin_update_list();
+
+	if (list_changed)
+	{
+		playlist_generate_shuffle_list();
+		playlistwin_update_list();
+	}
 }
 
 void playlist_start_get_info_thread(void)
@@ -1246,4 +1497,3 @@ void playlist_start_get_info_scan(void)
 {
 	playlist_get_info_scan_active = TRUE;
 }
-
