@@ -1,5 +1,5 @@
 /*  XMMS - Cross-platform multimedia player
- *  Copyright (C) 1998-1999  Peter Alm, Mikael Alm, Olle Hallnas, Thomas Nilsson and 4Front Technologies
+ *  Copyright (C) 1998-2000  Peter Alm, Mikael Alm, Olle Hallnas, Thomas Nilsson and 4Front Technologies
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -18,10 +18,9 @@
 #include "xmms.h"
 #include "libxmms/dirbrowser.h"
 
-GtkWidget *playlistwin, *playlistwin_add_filesel = NULL, *playlistwin_save_filesel = NULL;
+GtkWidget *playlistwin, *playlistwin_save_filesel = NULL;
 GtkWidget *playlistwin_load_filesel = NULL, *playlistwin_url_window = NULL,
          *playlistwin_dir_browser;
-GtkWidget *playlistwin_filesel_add_selected, *playlistwin_filesel_add_all;
 GtkItemFactory *playlistwin_sort_menu, *playlistwin_sub_menu;
 
 GdkPixmap *playlistwin_bg;
@@ -46,11 +45,6 @@ Vis *playlistwin_vis;
 
 static gboolean playlistwin_vis_enabled = FALSE;
 
-static GtkTargetEntry drop_types[] =
-{
-	{"text/plain", 0, 1}
-};
-
 enum
 {
 	ADD_URL, ADD_DIR, ADD_FILE
@@ -86,17 +80,17 @@ enum
 
 GtkItemFactoryEntry playlistwin_sort_menu_entries[] =
 {
-	{"/Sort List", NULL, NULL, 0, "<Branch>"},
-	{"/Sort List/By Title", NULL, playlistwin_sort_menu_callback, PLAYLISTWIN_SORT_BYTITLE, "<Item>"},
-	{"/Sort List/By Filename", NULL, playlistwin_sort_menu_callback, PLAYLISTWIN_SORT_BYFILENAME, "<Item>"},
-	{"/Sort List/By Path + Filename", NULL, playlistwin_sort_menu_callback, PLAYLISTWIN_SORT_BYPATH, "<Item>"},
-	{"/Sort Selection", NULL, NULL, 0, "<Branch>"},
-	{"/Sort Selection/By Title", NULL, playlistwin_sort_menu_callback, PLAYLISTWIN_SORT_SEL_BYTITLE, "<Item>"},
-	{"/Sort Selection/By Filename", NULL, playlistwin_sort_menu_callback, PLAYLISTWIN_SORT_SEL_BYFILENAME, "<Item>"},
-	{"/Sort Selection/By Path + Filename", NULL, playlistwin_sort_menu_callback, PLAYLISTWIN_SORT_SEL_BYPATH, "<Item>"},
-	{"/-", NULL, NULL, 0, "<Separator>"},
-	{"/Randomize List", NULL, playlistwin_sort_menu_callback, PLAYLISTWIN_SORT_RANDOMIZE, "<Item>"},
-	{"/Reverse List", NULL, playlistwin_sort_menu_callback, PLAYLISTWIN_SORT_REVERSE, "<Item>"},
+	{N_("/Sort List"), NULL, NULL, 0, "<Branch>"},
+	{N_("/Sort List/By Title"), NULL, playlistwin_sort_menu_callback, PLAYLISTWIN_SORT_BYTITLE, "<Item>"},
+	{N_("/Sort List/By Filename"), NULL, playlistwin_sort_menu_callback, PLAYLISTWIN_SORT_BYFILENAME, "<Item>"},
+	{N_("/Sort List/By Path + Filename"), NULL, playlistwin_sort_menu_callback, PLAYLISTWIN_SORT_BYPATH, "<Item>"},
+	{N_("/Sort Selection"), NULL, NULL, 0, "<Branch>"},
+	{N_("/Sort Selection/By Title"), NULL, playlistwin_sort_menu_callback, PLAYLISTWIN_SORT_SEL_BYTITLE, "<Item>"},
+	{N_("/Sort Selection/By Filename"), NULL, playlistwin_sort_menu_callback, PLAYLISTWIN_SORT_SEL_BYFILENAME, "<Item>"},
+	{N_("/Sort Selection/By Path + Filename"), NULL, playlistwin_sort_menu_callback, PLAYLISTWIN_SORT_SEL_BYPATH, "<Item>"},
+	{N_("/-"), NULL, NULL, 0, "<Separator>"},
+	{N_("/Randomize List"), NULL, playlistwin_sort_menu_callback, PLAYLISTWIN_SORT_RANDOMIZE, "<Item>"},
+	{N_("/Reverse List"), NULL, playlistwin_sort_menu_callback, PLAYLISTWIN_SORT_REVERSE, "<Item>"},
 };
 
 enum
@@ -108,7 +102,7 @@ enum
 
 GtkItemFactoryEntry playlistwin_sub_menu_entries[] =
 {
-	{"/Remove Dead Files", NULL, playlistwin_sub_menu_callback, PLAYLISTWIN_REMOVE_DEAD_FILES, "<Item>"},
+	{N_("/Remove Dead Files"), NULL, playlistwin_sub_menu_callback, PLAYLISTWIN_REMOVE_DEAD_FILES, "<Item>"},
 };
 
 void playlistwin_draw_frame(void);
@@ -185,21 +179,15 @@ void playlistwin_update_sinfo(void)
 		return;
 	}
 
-	entry = g_list_nth(playlist, get_playlist_position())->data;
+	entry = g_list_nth(playlist, __get_playlist_position())->data;
 
 	title = entry->title;
 	if (!title)
-	{
-		title = strrchr(entry->filename, '/');
-		if (!title)
-			title = entry->filename;
-		else
-			title++;
-	}
+		title = g_basename(entry->filename);
 
 	max_len = (cfg.playlist_width - 35) / 5;
 
-	pos = g_strdup_printf("%d. ", get_playlist_position() + 1);
+	pos = g_strdup_printf("%d. ", __get_playlist_position() + 1);
 
 	if (entry->length != -1)
 	{
@@ -317,7 +305,7 @@ void playlistwin_set_shade(gboolean shaded)
 		playlistwin_close->pb_ny = 3;
 	}
 	cfg.playlist_shaded = shaded;
-	GTK_CHECK_MENU_ITEM(gtk_item_factory_get_widget(mainwin_options_menu, "/Playlist WindowShade Mode"))->active = shaded;
+	GTK_CHECK_MENU_ITEM(gtk_item_factory_get_widget(mainwin_options_menu, _("/Playlist WindowShade Mode")))->active = shaded;
 
 	playlistwin_create_mask();
 	draw_playlist_window(TRUE);
@@ -524,6 +512,19 @@ void playlistwin_motion(GtkWidget * widget, GdkEventMotion * event, gpointer cal
 	while (XCheckMaskEvent(GDK_DISPLAY(), ButtonMotionMask, &ev)) ;
 }
 
+void playlistwin_show_filebrowser(void)
+{
+	static GtkWidget *filebrowser;
+	if (filebrowser != NULL)
+		return;
+
+	filebrowser = util_create_filebrowser(FALSE);
+	gtk_signal_connect(GTK_OBJECT(filebrowser), "destroy",
+			   GTK_SIGNAL_FUNC(gtk_widget_destroyed), &filebrowser);
+}
+
+
+
 void playlistwin_url_ok_clicked(GtkWidget * w, GtkWidget * entry)
 {
 	gchar *text, *temp;
@@ -538,7 +539,6 @@ void playlistwin_url_ok_clicked(GtkWidget * w, GtkWidget * entry)
 			temp = g_strdup(text);
 		playlist_add_url_string(temp);
 		g_free(temp);
-		playlist_generate_shuffle_list();
 		playlistwin_update_list();
 	}
 	gtk_widget_destroy(playlistwin_url_window);
@@ -548,7 +548,7 @@ void playlistwin_show_add_url_window(void)
 {
 	if(!playlistwin_url_window)
 	{
-		playlistwin_url_window = util_create_add_url_window("Enter URL to add:", GTK_SIGNAL_FUNC(playlistwin_url_ok_clicked));
+		playlistwin_url_window = util_create_add_url_window(_("Enter URL to add:"), GTK_SIGNAL_FUNC(playlistwin_url_ok_clicked), NULL);
 		gtk_window_set_transient_for(GTK_WINDOW(playlistwin_url_window), GTK_WINDOW(playlistwin));
 		gtk_signal_connect(GTK_OBJECT(playlistwin_url_window), "destroy", GTK_SIGNAL_FUNC(gtk_widget_destroyed), &playlistwin_url_window);
 		gtk_widget_show(playlistwin_url_window);
@@ -558,207 +558,17 @@ void playlistwin_show_add_url_window(void)
 
 void playlistwin_add_dir_handler(gchar * dir)
 {
-	if (cfg.filesel_path)
-		g_free(cfg.filesel_path);
+	g_free(cfg.filesel_path);
 	cfg.filesel_path = g_strdup(dir);
 	playlist_add_dir(dir);
 	playlistwin_update_list();
 }
 
-static int int_compare_func(const void *a, const void *b)
-{
-	if ((gint) a < (gint) b)
-		return -1;
-	if ((gint) a > (gint) b)
-		return 1;
-	else
-		return 0;
-}
-
-void playlistwin_add_filesel_changed(GtkWidget * w, GtkFileSelection * filesel)
-{
-	gchar *text = gtk_entry_get_text(GTK_ENTRY(w)), *current = "./",
-	     *parent = "../";
-	GList *list, *node;
-
-	if (!text || !(*text))
-	{
-		if ((list = input_scan_dir(gtk_file_selection_get_filename(GTK_FILE_SELECTION(filesel)))) != NULL)
-		{
-			/* We enter a directory that has been "hijacked" by a input-plugin
-			   This is used by the CDDA plugin */
-			gtk_clist_clear(GTK_CLIST(filesel->dir_list));
-			gtk_clist_append(GTK_CLIST(filesel->dir_list), &current);
-			gtk_clist_append(GTK_CLIST(filesel->dir_list), &parent);
-
-			gtk_clist_freeze(GTK_CLIST(filesel->file_list));
-			gtk_clist_clear(GTK_CLIST(filesel->file_list));
-			node = list;
-			while (node)
-			{
-				gtk_clist_append(GTK_CLIST(filesel->file_list), (gchar **) & node->data);
-				g_free(node->data);
-				node = g_list_next(node);
-			}
-			gtk_clist_thaw(GTK_CLIST(filesel->file_list));
-			g_list_free(list);
-		}
-	}
-}
-
-static gboolean playlistwin_is_dir(GtkFileSelection * filesel)
-{
-	gchar *text;
-	struct stat buf;
-	
-	text = g_strdup(gtk_file_selection_get_filename(filesel));
-	g_strstrip(text);
-	while ((text[strlen(text) - 1] == '/') && (strlen(text) > 1) && (text[strlen(text) - 2] == '/'))
-	{
-		text[strlen(text) - 1] = '\0';
-	}
-
-	if (stat(text, &buf) == 0)
-	{
-		if (S_ISDIR(buf.st_mode))
-		{		/* Selected directory */
-			gtk_file_selection_set_filename(filesel, text);
-			g_free(text);
-			return TRUE;
-		}
-	}
-	g_free(text);
-	return FALSE;
-}
-
-static void playlistwin_add_files(GtkFileSelection * filesel)
-{
-	GList *sel_list = NULL, *node;
-	gchar *text, *text2;
-
-
-	if (cfg.filesel_path)
-		g_free(cfg.filesel_path);
-	cfg.filesel_path = g_strdup(gtk_file_selection_get_filename(filesel));
-	if(!cfg.filesel_path)
-		return;
-	
-	text = strrchr(cfg.filesel_path, '/');
-	if (text)
-		*(text + 1) = '\0';
-
-	node = GTK_CLIST(filesel->file_list)->selection;
-	while(node)
-	{
-		sel_list = g_list_prepend(sel_list, node->data);
-		node = g_list_next(node);
-	}
-	sel_list = g_list_sort(sel_list, int_compare_func);
-
-	if(cfg.open_rev_order)
-		node = g_list_last(sel_list);
-	else
-		node = sel_list;
-	
-	if (node)
-	{
-		while (node)
-		{
-			gtk_clist_get_text(GTK_CLIST(filesel->file_list), GPOINTER_TO_INT(node->data), 0, &text);
-			text2 = g_strconcat(cfg.filesel_path, text, NULL);
-			playlist_add(text2);
-			g_free(text2);
-			if(cfg.open_rev_order)
-				node = g_list_previous(node);
-			else
-				node = g_list_next(node);
-		}
-		playlist_generate_shuffle_list();
-	}
-/*
-	else
-	{
-		text = gtk_file_selection_get_filename(filesel);
-		if (text[strlen(text) - 1] != '/')
-			playlist_add(text);
-		playlist_generate_shuffle_list();
-	}
-*/
-	g_list_free(sel_list);
-	playlistwin_update_list();
-}
-
-void playlistwin_add_filesel_ok(GtkWidget * w, GtkWidget * filesel)
-{
-	if(!playlistwin_is_dir(GTK_FILE_SELECTION(filesel)))
-	{
-		gtk_widget_hide(filesel);
-		playlistwin_add_files(GTK_FILE_SELECTION(filesel));
-		gtk_widget_destroy(filesel);
-	}
-}
-
-void playlistwin_add_selected_files(GtkWidget * w, GtkWidget * filesel)
-{
-	playlistwin_add_files(GTK_FILE_SELECTION(filesel));
-}
-
-void playlistwin_add_all_files(GtkWidget * w, GtkWidget * filesel)
-{
-	if(GTK_CLIST(GTK_FILE_SELECTION(filesel)->file_list)->row_list)
-	{
-		gtk_clist_freeze(GTK_CLIST(GTK_FILE_SELECTION(filesel)->file_list));
-		gtk_clist_select_all(GTK_CLIST(GTK_FILE_SELECTION(filesel)->file_list));
-		playlistwin_add_files(GTK_FILE_SELECTION(filesel));
-		/*
-		 * We want gtk_clist_undo_selection() but it is buggy in GTK+ 1.2.6
-		 */
-		/*  gtk_clist_undo_selection(GTK_CLIST(GTK_FILE_SELECTION(filesel)->file_list)); */
-		gtk_clist_unselect_all(GTK_CLIST(GTK_FILE_SELECTION(filesel)->file_list));
-		gtk_clist_thaw(GTK_CLIST(GTK_FILE_SELECTION(filesel)->file_list));
-	}
-}
-
-void playlistwin_show_filebrowser(void)
-{
-	GtkWidget *bbox;
-	if (!playlistwin_add_filesel)
-	{
-		playlistwin_add_filesel = gtk_file_selection_new("Add file(s) to playlist");
-		
-		gtk_clist_set_selection_mode(GTK_CLIST(GTK_FILE_SELECTION(playlistwin_add_filesel)->file_list), GTK_SELECTION_EXTENDED);
-		gtk_signal_connect(GTK_OBJECT(GTK_FILE_SELECTION(playlistwin_add_filesel)->selection_entry), "changed", GTK_SIGNAL_FUNC(playlistwin_add_filesel_changed), playlistwin_add_filesel);
-		gtk_signal_connect(GTK_OBJECT(GTK_FILE_SELECTION(playlistwin_add_filesel)->ok_button), "clicked", GTK_SIGNAL_FUNC(playlistwin_add_filesel_ok), playlistwin_add_filesel);
-		gtk_signal_connect_object(GTK_OBJECT(GTK_FILE_SELECTION(playlistwin_add_filesel)->cancel_button), "clicked", GTK_SIGNAL_FUNC(gtk_widget_destroy), GTK_OBJECT(playlistwin_add_filesel));
-		gtk_signal_connect(GTK_OBJECT(playlistwin_add_filesel), "destroy", GTK_SIGNAL_FUNC(gtk_widget_destroyed), &playlistwin_add_filesel);
-		if (cfg.filesel_path)
-			gtk_file_selection_set_filename(GTK_FILE_SELECTION(playlistwin_add_filesel), cfg.filesel_path);
-		bbox = gtk_hbutton_box_new();
-		gtk_button_box_set_layout(GTK_BUTTON_BOX(bbox), GTK_BUTTONBOX_END);
-		gtk_button_box_set_spacing(GTK_BUTTON_BOX(bbox), 0);
-		gtk_box_pack_end(GTK_BOX(GTK_FILE_SELECTION(playlistwin_add_filesel)->action_area), bbox, TRUE, TRUE, 0);
-		playlistwin_filesel_add_selected  = gtk_button_new_with_label("Add selected files");
-		gtk_box_pack_start(GTK_BOX(bbox), playlistwin_filesel_add_selected, FALSE, FALSE, 0);
-		gtk_widget_show(playlistwin_filesel_add_selected);
-		gtk_signal_connect(GTK_OBJECT(playlistwin_filesel_add_selected), "clicked", GTK_SIGNAL_FUNC(playlistwin_add_selected_files), playlistwin_add_filesel);
-		playlistwin_filesel_add_all = gtk_button_new_with_label("Add all files in directory");
-		gtk_box_pack_start(GTK_BOX(bbox), playlistwin_filesel_add_all, FALSE, FALSE, 0);
-		gtk_widget_show(playlistwin_filesel_add_all);
-		gtk_signal_connect(GTK_OBJECT(playlistwin_filesel_add_all), "clicked", GTK_SIGNAL_FUNC(playlistwin_add_all_files), playlistwin_add_filesel);
-		gtk_widget_show(bbox);
-
-		gtk_widget_show(playlistwin_add_filesel);
-		
-	}
-	if (!GTK_WIDGET_VISIBLE(playlistwin_add_filesel))
-		gtk_widget_show(playlistwin_add_filesel);
-}
-
 void playlistwin_show_dirbrowser(void)
 {
 	if (!playlistwin_dir_browser)
 	{
-		playlistwin_dir_browser = xmms_create_dir_browser("Select directory to add:", cfg.filesel_path, GTK_SELECTION_EXTENDED, playlistwin_add_dir_handler);
+		playlistwin_dir_browser = xmms_create_dir_browser(_("Select directory to add:"), cfg.filesel_path, GTK_SELECTION_EXTENDED, playlistwin_add_dir_handler);
 		gtk_signal_connect(GTK_OBJECT(playlistwin_dir_browser), "destroy", GTK_SIGNAL_FUNC(gtk_widget_destroyed), &playlistwin_dir_browser);
 		gtk_window_set_transient_for(GTK_WINDOW(playlistwin_dir_browser), GTK_WINDOW(playlistwin));
 	}
@@ -826,10 +636,14 @@ void playlistwin_sel_popup_handler(gint item)
 
 static void playlistwin_set_sensitive_sortmenu(gboolean set)
 {
-	gtk_widget_set_sensitive(gtk_item_factory_get_widget(playlistwin_sort_menu, "/Sort Selection/By Title"),set);
-	gtk_widget_set_sensitive(gtk_item_factory_get_widget(playlistwin_sort_menu, "/Sort Selection/By Filename"),set);
-	gtk_widget_set_sensitive(gtk_item_factory_get_widget(playlistwin_sort_menu, "/Sort Selection/By Path + Filename"),set);
-	/* gtk_widget_set_sensitive(gtk_item_factory_get_widget(playlistwin_sort_menu, "/Sort Selection"),set); */
+	gtk_widget_set_sensitive(gtk_item_factory_get_widget(playlistwin_sort_menu, _("/Sort Selection/By Title")),set);
+	gtk_widget_set_sensitive(gtk_item_factory_get_widget(playlistwin_sort_menu, _("/Sort Selection/By Filename")),set);
+	gtk_widget_set_sensitive(gtk_item_factory_get_widget(playlistwin_sort_menu, _("/Sort Selection/By Path + Filename")),set);
+
+#if 0
+	gtk_widget_set_sensitive(gtk_item_factory_get_widget(playlistwin_sort_menu, _("/Sort Selection")),set);
+#endif
+
 }
 
 void playlistwin_misc_popup_handler(gint item)
@@ -891,27 +705,20 @@ void playlistwin_misc_popup_handler(gint item)
 
 void playlistwin_save_filesel_ok(GtkWidget * w, GtkWidget * filesel)
 {
-	gchar *filename, *text, *text2;
-	struct stat buf;
+	gchar *filename, *text, *tmp;
 
-	text = g_strdup(gtk_file_selection_get_filename(GTK_FILE_SELECTION(filesel)));
-	while (*(strrchr(text, '/') + 1) == '\0')
-		*(strrchr(text, '/')) = '\0';
-
-	if (stat(text, &buf) == 0)
-	{
-		if (S_ISDIR(buf.st_mode))
-		{		/* Selected directory -- don't close frequester */
-			text2 = g_strdup_printf("%s/", text);
-			gtk_file_selection_set_filename(GTK_FILE_SELECTION(filesel), text2);
-			g_free(text2);
-			return;
-		}
-	}
+	if (util_filebrowser_is_dir(GTK_FILE_SELECTION(filesel)))
+		return;
+	
+	filename = g_strdup(gtk_file_selection_get_filename(GTK_FILE_SELECTION(filesel)));
+	text = g_strdup(filename);
 
+	if ((tmp = strrchr(text, '/')) != NULL)
+		*(tmp + 1) = '\0';
+	g_free(cfg.playlist_path);
+	cfg.playlist_path = g_strdup(text);
 	g_free(text);
 
-	filename = gtk_file_selection_get_filename(GTK_FILE_SELECTION(filesel));
 	if (filename && *filename)
 		playlist_save(filename);
 	gtk_widget_destroy(GTK_WIDGET(filesel));
@@ -919,27 +726,20 @@ void playlistwin_save_filesel_ok(GtkWidget * w, GtkWidget * filesel)
 
 void playlistwin_load_filesel_ok(GtkWidget * w, GtkWidget * filesel)
 {
-	gchar *filename, *text, *text2;
-	struct stat buf;
+	gchar *filename, *text, *tmp;
 
-	text = g_strdup(gtk_file_selection_get_filename(GTK_FILE_SELECTION(filesel)));
-	while (*(strrchr(text, '/') + 1) == '\0')
-		*(strrchr(text, '/')) = '\0';
-
-	if (stat(text, &buf) == 0)
-	{
-		if (S_ISDIR(buf.st_mode))
-		{		/* Selected directory -- don't close frequester */
-			text2 = g_strdup_printf("%s/", text);
-			gtk_file_selection_set_filename(GTK_FILE_SELECTION(filesel), text2);
-			g_free(text2);
-			return;
-		}
-	}
+	if (util_filebrowser_is_dir(GTK_FILE_SELECTION(filesel)))
+		return;
+	
+	filename = g_strdup(gtk_file_selection_get_filename(GTK_FILE_SELECTION(filesel)));
+	text = g_strdup(filename);
 
+	if ((tmp = strrchr(text, '/')) != NULL)
+		*(tmp + 1) = '\0';
+	g_free(cfg.playlist_path);
+	cfg.playlist_path = g_strdup(text);
 	g_free(text);
 
-	filename = gtk_file_selection_get_filename(GTK_FILE_SELECTION(filesel));
 	if (filename && *filename)
 	{
 		playlist_clear();
@@ -963,31 +763,36 @@ void playlistwin_plist_popup_handler(gint item)
 			mainwin_set_song_info(0, 0, 0);
 			mainwin_set_info_text();
 			break;
+
 		case PLIST_SAVE:
-			if (!playlistwin_save_filesel)
-			{
-				playlistwin_save_filesel = gtk_file_selection_new("Save playlist");
-				gtk_signal_connect(GTK_OBJECT(GTK_FILE_SELECTION(playlistwin_save_filesel)->ok_button), "clicked", GTK_SIGNAL_FUNC(playlistwin_save_filesel_ok), playlistwin_save_filesel);
-				gtk_signal_connect_object(GTK_OBJECT(GTK_FILE_SELECTION(playlistwin_save_filesel)->cancel_button), "clicked", GTK_SIGNAL_FUNC(gtk_widget_destroy), GTK_OBJECT(playlistwin_save_filesel));
-				gtk_signal_connect(GTK_OBJECT(playlistwin_save_filesel), "destroy", GTK_SIGNAL_FUNC(gtk_widget_destroyed), &playlistwin_save_filesel);
-				gtk_widget_show(playlistwin_save_filesel);
-			}
-			if (!GTK_WIDGET_VISIBLE(playlistwin_save_filesel))
-				gtk_widget_show(playlistwin_save_filesel);
+			if (playlistwin_save_filesel != NULL)
+				break;
 
+			playlistwin_save_filesel = gtk_file_selection_new(_("Save playlist"));
+			if (cfg.playlist_path)
+				gtk_file_selection_set_filename(GTK_FILE_SELECTION(playlistwin_save_filesel), cfg.playlist_path);
+			gtk_signal_connect(GTK_OBJECT(GTK_FILE_SELECTION(playlistwin_save_filesel)->ok_button), "clicked", GTK_SIGNAL_FUNC(playlistwin_save_filesel_ok), playlistwin_save_filesel);
+			gtk_signal_connect_object(GTK_OBJECT(GTK_FILE_SELECTION(playlistwin_save_filesel)->cancel_button), "clicked", GTK_SIGNAL_FUNC(gtk_widget_destroy), GTK_OBJECT(playlistwin_save_filesel));
+			gtk_signal_connect(GTK_OBJECT(playlistwin_save_filesel),
+					   "destroy",
+					   GTK_SIGNAL_FUNC(gtk_widget_destroyed),
+					   &playlistwin_save_filesel);
+			gtk_widget_show(playlistwin_save_filesel);
 			break;
-		case PLIST_LOAD:
-			if (!playlistwin_load_filesel)
-			{
-				playlistwin_load_filesel = gtk_file_selection_new("Load playlist");
-				gtk_signal_connect(GTK_OBJECT(GTK_FILE_SELECTION(playlistwin_load_filesel)->ok_button), "clicked", GTK_SIGNAL_FUNC(playlistwin_load_filesel_ok), playlistwin_load_filesel);
-				gtk_signal_connect_object(GTK_OBJECT(GTK_FILE_SELECTION(playlistwin_load_filesel)->cancel_button), "clicked", GTK_SIGNAL_FUNC(gtk_widget_destroy), GTK_OBJECT(playlistwin_load_filesel));
-				gtk_signal_connect(GTK_OBJECT(playlistwin_load_filesel), "destroy", GTK_SIGNAL_FUNC(gtk_widget_destroyed), &playlistwin_load_filesel);
-				gtk_widget_show(playlistwin_load_filesel);
-			}
-			if (!GTK_WIDGET_VISIBLE(playlistwin_load_filesel))
-				gtk_widget_show(playlistwin_load_filesel);
 
+		case PLIST_LOAD:
+			if (playlistwin_load_filesel != NULL)
+				break;
+			playlistwin_load_filesel = gtk_file_selection_new(_("Load playlist"));
+			if (cfg.playlist_path)
+				gtk_file_selection_set_filename(GTK_FILE_SELECTION(playlistwin_load_filesel), cfg.playlist_path);
+			gtk_signal_connect(GTK_OBJECT(GTK_FILE_SELECTION(playlistwin_load_filesel)->ok_button), "clicked", GTK_SIGNAL_FUNC(playlistwin_load_filesel_ok), playlistwin_load_filesel);
+			gtk_signal_connect_object(GTK_OBJECT(GTK_FILE_SELECTION(playlistwin_load_filesel)->cancel_button), "clicked", GTK_SIGNAL_FUNC(gtk_widget_destroy), GTK_OBJECT(playlistwin_load_filesel));
+			gtk_signal_connect(GTK_OBJECT(playlistwin_load_filesel),
+					   "destroy",
+					   GTK_SIGNAL_FUNC(gtk_widget_destroyed),
+					   &playlistwin_load_filesel);
+			gtk_widget_show(playlistwin_load_filesel);
 			break;
 	}
 }
@@ -1054,7 +859,12 @@ void playlistwin_press(GtkWidget * widget, GdkEventButton * event, gpointer call
 
 	gboolean grab = TRUE;
 
-	if (event->button == 1 && ((!cfg.playlist_shaded && event->x > cfg.playlist_width - 20 && event->y > cfg.playlist_height - 20) || (cfg.playlist_shaded && event->x >= cfg.playlist_width - 31 && event->x < cfg.playlist_width - 22)))
+	if (event->button == 1 &&
+	    ((!cfg.playlist_shaded && event->x > cfg.playlist_width - 20 &&
+	      event->y > cfg.playlist_height - 20) ||
+	     (cfg.playlist_shaded &&
+	      event->x >= cfg.playlist_width - 31 &&
+	      event->x < cfg.playlist_width - 22)))
 	{
 		playlistwin_resizing = TRUE;
 		playlistwin_resize_x = cfg.playlist_width - event->x;
@@ -1064,36 +874,46 @@ void playlistwin_press(GtkWidget * widget, GdkEventButton * event, gpointer call
 	else if (event->button == 1 && REGION_L(12, 37, 29, 11))
 	{
 		dock_get_widget_pos(playlistwin, &cfg.playlist_x, &cfg.playlist_y);
-		playlist_popup(cfg.playlist_x + 12, cfg.playlist_y + cfg.playlist_height - (3 * 18) - 11, 3, add_nx, add_ny,
-			       add_sx, add_sy, add_barx, add_bary, playlistwin_add_popup_handler);
+		playlist_popup(cfg.playlist_x + 12,
+			       cfg.playlist_y + cfg.playlist_height - (3 * 18) - 11,
+			       3, add_nx, add_ny, add_sx, add_sy, add_barx, add_bary,
+			       playlistwin_add_popup_handler);
 		grab = FALSE;
 	}
 	else if (event->button == 1 && REGION_L(41, 66, 29, 11))
 	{
-		dock_get_widget_pos(playlistwin, &cfg.playlist_x, &cfg.playlist_y);		
-		playlist_popup(cfg.playlist_x + 41, cfg.playlist_y + cfg.playlist_height - (4 * 18) - 11, 4, sub_nx, sub_ny,
-			       sub_sx, sub_sy, sub_barx, sub_bary, playlistwin_sub_popup_handler);
+		dock_get_widget_pos(playlistwin, &cfg.playlist_x, &cfg.playlist_y);
+		playlist_popup(cfg.playlist_x + 41,
+			       cfg.playlist_y + cfg.playlist_height - (4 * 18) - 11,
+			       4, sub_nx, sub_ny, sub_sx, sub_sy, sub_barx, sub_bary,
+			       playlistwin_sub_popup_handler);
 		grab = FALSE;
 	}
 	else if (event->button == 1 && REGION_L(70, 95, 29, 11))
 	{
-		dock_get_widget_pos(playlistwin, &cfg.playlist_x, &cfg.playlist_y);		
-		playlist_popup(cfg.playlist_x + 70, cfg.playlist_y + cfg.playlist_height - (3 * 18) - 11, 3, sel_nx, sel_ny,
-			       sel_sx, sel_sy, sel_barx, sel_bary, playlistwin_sel_popup_handler);
+		dock_get_widget_pos(playlistwin, &cfg.playlist_x, &cfg.playlist_y);
+		playlist_popup(cfg.playlist_x + 70,
+			       cfg.playlist_y + cfg.playlist_height - (3 * 18) - 11,
+			       3, sel_nx, sel_ny, sel_sx, sel_sy, sel_barx, sel_bary,
+			       playlistwin_sel_popup_handler);
 		grab = FALSE;
 	}
 	else if (event->button == 1 && REGION_L(99, 124, 29, 11))
 	{
-		dock_get_widget_pos(playlistwin, &cfg.playlist_x, &cfg.playlist_y);		
-		playlist_popup(cfg.playlist_x + 99, cfg.playlist_y + cfg.playlist_height - (3 * 18) - 11, 3, misc_nx, misc_ny,
-			       misc_sx, misc_sy, misc_barx, misc_bary, playlistwin_misc_popup_handler);
+		dock_get_widget_pos(playlistwin, &cfg.playlist_x, &cfg.playlist_y);
+		playlist_popup(cfg.playlist_x + 99,
+			       cfg.playlist_y + cfg.playlist_height - (3 * 18) - 11,
+			       3, misc_nx, misc_ny, misc_sx, misc_sy, misc_barx,
+			       misc_bary, playlistwin_misc_popup_handler);
 		grab = FALSE;
 	}
 	else if (event->button == 1 && REGION_R(46, 23, 29, 11))
 	{
-		dock_get_widget_pos(playlistwin, &cfg.playlist_x, &cfg.playlist_y);		
-		playlist_popup(cfg.playlist_x + cfg.playlist_width - 46, cfg.playlist_y + cfg.playlist_height - (3 * 18) - 11, 3, plist_nx, plist_ny,
-			       plist_sx, plist_sy, plist_barx, plist_bary, playlistwin_plist_popup_handler);
+		dock_get_widget_pos(playlistwin, &cfg.playlist_x, &cfg.playlist_y);
+		playlist_popup(cfg.playlist_x + cfg.playlist_width - 46,
+			       cfg.playlist_y + cfg.playlist_height - (3 * 18) - 11,
+			       3, plist_nx, plist_ny, plist_sx, plist_sy, plist_barx,
+			       plist_bary, playlistwin_plist_popup_handler);
 		grab = FALSE;
 	}
 	else if (event->button == 1 && REGION_R(82, 54, 15, 9))
@@ -1122,13 +942,16 @@ void playlistwin_press(GtkWidget * widget, GdkEventButton * event, gpointer call
 			grab = FALSE;
 		}
 	}
-	else if (event->button == 1 && event->type == GDK_BUTTON_PRESS && !inside_sensitive_widgets(event->x, event->y) && (cfg.easy_move || event->y < 14))
+	else if (event->button == 1 && event->type == GDK_BUTTON_PRESS &&
+		 !inside_sensitive_widgets(event->x, event->y) &&
+		 (cfg.easy_move || event->y < 14))
 	{
 		gdk_window_raise(playlistwin->window);
 		dock_move_press(dock_window_list, playlistwin, event, FALSE);
 
 	}
-	else if (event->button == 1 && event->type == GDK_2BUTTON_PRESS && !inside_sensitive_widgets(event->x, event->y) && event->y < 14)
+	else if (event->button == 1 && event->type == GDK_2BUTTON_PRESS &&
+		 !inside_sensitive_widgets(event->x, event->y) && event->y < 14)
 	{
 		playlistwin_set_shade(!cfg.playlist_shaded);
 		if(dock_is_moving(playlistwin))
@@ -1143,16 +966,13 @@ void playlistwin_press(GtkWidget * widget, GdkEventButton * event, gpointer call
 		     (event->x >= 70 && event->x < 95) ||
 		     (event->x >= 99 && event->x < 124) ||
 		     (event->x >= cfg.playlist_width - 46 &&
-		      event->x < cfg.playlist_width - 23)
-				)
-			)
-		))
+		      event->x < cfg.playlist_width - 23)))))
 	{
-		gint mx, my;
-		GdkModifierType modmask;
-
-		gdk_window_get_pointer(NULL, &mx, &my, &modmask);
-		util_item_factory_popup(mainwin_general_menu, mx, my, 3, GDK_CURRENT_TIME);
+		/*
+		 * Pop up the main menu a few pixels down to avoid
+		 * anything to be selected initially.
+		 */
+		util_item_factory_popup(mainwin_general_menu, event->x_root, event->y_root + 2, 3, GDK_CURRENT_TIME);
 		grab = FALSE;
 	}
 	else if (event->button == 4)
@@ -1165,7 +985,9 @@ void playlistwin_press(GtkWidget * widget, GdkEventButton * event, gpointer call
 		draw_playlist_window(FALSE);
 	}
 	if (grab)
-		gdk_pointer_grab(playlistwin->window, FALSE, GDK_BUTTON_MOTION_MASK | GDK_BUTTON_RELEASE_MASK, GDK_NONE, GDK_NONE, GDK_CURRENT_TIME);
+		gdk_pointer_grab(playlistwin->window, FALSE,
+				 GDK_BUTTON_MOTION_MASK | GDK_BUTTON_RELEASE_MASK,
+				 GDK_NONE, GDK_NONE, GDK_CURRENT_TIME);
 
 }
 
@@ -1185,9 +1007,12 @@ void playlistwin_focus_out(GtkWidget * widget, GdkEventButton * event, gpointer
 	draw_playlist_window(TRUE);
 }
 
-gint playlistwin_configure(GtkWidget * window)
+static gboolean playlistwin_configure(GtkWidget * window, GdkEventConfigure *event, gpointer data)
 {
-	gdk_window_get_deskrelative_origin(playlistwin->window, &cfg.playlist_x, &cfg.playlist_y);
+	if (!GTK_WIDGET_VISIBLE(window))
+		return FALSE;
+	
+	gdk_window_get_deskrelative_origin(window->window, &cfg.playlist_x, &cfg.playlist_y);
 	return FALSE;
 }
 
@@ -1228,7 +1053,11 @@ gboolean playlistwin_keypress(GtkWidget * w, GdkEventKey * event, gpointer data)
 	gboolean refresh = TRUE;
 
 	if (cfg.playlist_shaded)
+	{
+		gtk_widget_event(mainwin, (GdkEvent *) event);
 		return TRUE;
+	}
+	
 	switch (keyval = event->keyval)
 	{
 		case GDK_KP_Up:
@@ -1351,6 +1180,7 @@ gboolean playlistwin_keypress(GtkWidget * w, GdkEventKey * event, gpointer data)
 			refresh=FALSE;
 			break;
 		default:
+			gtk_widget_event(mainwin, (GdkEvent *) event);
 			refresh = FALSE;
 			break;
 	}
@@ -1363,19 +1193,19 @@ gboolean playlistwin_keypress(GtkWidget * w, GdkEventKey * event, gpointer data)
 void playlistwin_draw_frame(void)
 {
 	gint w, h, y, i, c;
-	GdkPixmap *src;
+	SkinIndex src;
 
 	w = cfg.playlist_width;
 	h = cfg.playlist_height;
-	src = get_skin_pixmap(SKIN_PLEDIT);
+	src = SKIN_PLEDIT;
 
 	if (cfg.playlist_shaded)
 	{
-		gdk_draw_pixmap(playlistwin_bg, playlistwin_gc, src, 72, 42, 0, 0, 25, 14);
+		skin_draw_pixmap(playlistwin_bg, playlistwin_gc, src, 72, 42, 0, 0, 25, 14);
 		c = (w - 75) / 25;
 		for (i = 0; i < c; i++)
-			gdk_draw_pixmap(playlistwin_bg, playlistwin_gc, src, 72, 57, (i * 25) + 25, 0, 25, 14);
-		gdk_draw_pixmap(playlistwin_bg, playlistwin_gc, src, 99, (!playlistwin_focus && cfg.dim_titlebar) ? 57 : 42, w - 50, 0, 50, 14);
+			skin_draw_pixmap(playlistwin_bg, playlistwin_gc, src, 72, 57, (i * 25) + 25, 0, 25, 14);
+		skin_draw_pixmap(playlistwin_bg, playlistwin_gc, src, 99, (!playlistwin_focus && cfg.dim_titlebar) ? 57 : 42, w - 50, 0, 50, 14);
 	}
 	else
 	{
@@ -1383,37 +1213,46 @@ void playlistwin_draw_frame(void)
 			y = 0;
 		else
 			y = 21;
-		gdk_draw_pixmap(playlistwin_bg, playlistwin_gc, src, 0, y, 0, 0, 25, 20);
+		/* Titlebar left corner */
+		skin_draw_pixmap(playlistwin_bg, playlistwin_gc, src, 0, y, 0, 0, 25, 20);
 		c = (w - 150) / 25;
+		/* Titlebar, left and right of the title */
 		for (i = 0; i < c / 2; i++)
 		{
-			gdk_draw_pixmap(playlistwin_bg, playlistwin_gc, src, 127, y, (i * 25) + 25, 0, 25, 20);
-			gdk_draw_pixmap(playlistwin_bg, playlistwin_gc, src, 127, y, (i * 25) + (w / 2) + 50, 0, 25, 20);
+			skin_draw_pixmap(playlistwin_bg, playlistwin_gc, src, 127, y, (i * 25) + 25, 0, 25, 20);
+			skin_draw_pixmap(playlistwin_bg, playlistwin_gc, src, 127, y, (i * 25) + (w / 2) + 50, 0, 25, 20);
 		}
 		if (c & 1)
 		{
-			gdk_draw_pixmap(playlistwin_bg, playlistwin_gc, src, 127, y, ((c / 2) * 25) + 25, 0, 12, 20);
-			gdk_draw_pixmap(playlistwin_bg, playlistwin_gc, src, 127, y, (w / 2) + ((c / 2) * 25) + 50, 0, 13, 20);
+			skin_draw_pixmap(playlistwin_bg, playlistwin_gc, src, 127, y, ((c / 2) * 25) + 25, 0, 12, 20);
+			skin_draw_pixmap(playlistwin_bg, playlistwin_gc, src, 127, y, (w / 2) + ((c / 2) * 25) + 50, 0, 13, 20);
 		}
 
-		gdk_draw_pixmap(playlistwin_bg, playlistwin_gc, src, 26, y, (w / 2) - 50, 0, 100, 20);
-		gdk_draw_pixmap(playlistwin_bg, playlistwin_gc, src, 153, y, w - 25, 0, 25, 20);
+		/* Titlebar title */
+		skin_draw_pixmap(playlistwin_bg, playlistwin_gc, src, 26, y, (w / 2) - 50, 0, 100, 20);
+		/* Titlebar, right corner */
+		skin_draw_pixmap(playlistwin_bg, playlistwin_gc, src, 153, y, w - 25, 0, 25, 20);
 
+		/* Left and right side */
 		for (i = 0; i < (h - 58) / 29; i++)
 		{
-			gdk_draw_pixmap(playlistwin_bg, playlistwin_gc, src, 0, 42, 0, (i * 29) + 20, 12, 29);
-			gdk_draw_pixmap(playlistwin_bg, playlistwin_gc, src, 32, 42, w - 19, (i * 29) + 20, 19, 29);
+			skin_draw_pixmap(playlistwin_bg, playlistwin_gc, src, 0, 42, 0, (i * 29) + 20, 12, 29);
+			skin_draw_pixmap(playlistwin_bg, playlistwin_gc, src, 32, 42, w - 19, (i * 29) + 20, 19, 29);
 		}
-		gdk_draw_pixmap(playlistwin_bg, playlistwin_gc, src, 0, 72, 0, h - 38, 125, 38);
+		/* Bottom left corner (menu buttons) */
+		skin_draw_pixmap(playlistwin_bg, playlistwin_gc, src, 0, 72, 0, h - 38, 125, 38);
 		c = (w - 275) / 25;
+		/* Visualization window */
 		if (c >= 3)
 		{
 			c -= 3;
-			gdk_draw_pixmap(playlistwin_bg, playlistwin_gc, src, 205, 0, w - 225, h - 38, 75, 38);
+			skin_draw_pixmap(playlistwin_bg, playlistwin_gc, src, 205, 0, w - 225, h - 38, 75, 38);
 		}
+		/* Bottom blank parts */
 		for (i = 0; i < c; i++)
-			gdk_draw_pixmap(playlistwin_bg, playlistwin_gc, src, 179, 0, (i * 25) + 125, h - 38, 25, 38);
-		gdk_draw_pixmap(playlistwin_bg, playlistwin_gc, src, 126, 72, w - 150, h - 38, 150, 38);
+			skin_draw_pixmap(playlistwin_bg, playlistwin_gc, src, 179, 0, (i * 25) + 125, h - 38, 25, 38);
+		/* Bottom right corner (playbuttons etc) */
+		skin_draw_pixmap(playlistwin_bg, playlistwin_gc, src, 126, 72, w - 150, h - 38, 150, 38);
 	}
 }
 
@@ -1562,7 +1401,8 @@ static void playlistwin_drag_data_received(GtkWidget * widget,
 					   gint y,
 					   GtkSelectionData * selection_data,
 					   guint info,
-					   guint time)
+					   guint time,
+					   gpointer user_data)
 {
 	guint pos;
 
@@ -1588,7 +1428,7 @@ void playlistwin_create(void)
 	dock_add_window(dock_window_list, playlistwin);
 	gtk_widget_set_app_paintable(playlistwin, TRUE);
 	gtk_window_set_policy(GTK_WINDOW(playlistwin), FALSE, FALSE, TRUE);
-	gtk_window_set_title(GTK_WINDOW(playlistwin), "XMMS Playlist");
+	gtk_window_set_title(GTK_WINDOW(playlistwin), _("XMMS Playlist"));
 	gtk_window_set_wmclass(GTK_WINDOW(playlistwin), "XMMS_Playlist", "xmms");
 	gtk_window_set_transient_for(GTK_WINDOW(playlistwin), GTK_WINDOW(mainwin));
 	if (cfg.playlist_x != -1 && cfg.save_window_position)
@@ -1599,6 +1439,18 @@ void playlistwin_create(void)
 	hint_set_skip_winlist(playlistwin);
 
 	playlistwin_sort_menu = gtk_item_factory_new(GTK_TYPE_MENU, "<Main>", NULL);
+#ifdef ENABLE_NLS
+{
+	int i;
+	for (i=0; i < PLAYLISTWIN_SORT_MENU_ENTRIES; i++)
+	{
+		playlistwin_sort_menu_entries[i].path =
+			gettext(playlistwin_sort_menu_entries[i].path);
+	}
+	playlistwin_sub_menu_entries[0].path =
+		gettext(playlistwin_sub_menu_entries[0].path);
+}
+#endif
 	gtk_item_factory_create_items(GTK_ITEM_FACTORY(playlistwin_sort_menu), PLAYLISTWIN_SORT_MENU_ENTRIES, playlistwin_sort_menu_entries, NULL);
 	playlistwin_sub_menu = gtk_item_factory_new(GTK_TYPE_MENU, "<Main>", NULL);
 	gtk_item_factory_create_items(GTK_ITEM_FACTORY(playlistwin_sub_menu), PLAYLISTWIN_SUB_MENU_ENTRIES, playlistwin_sub_menu_entries, NULL);
@@ -1611,8 +1463,8 @@ void playlistwin_create(void)
 	gtk_signal_connect(GTK_OBJECT(playlistwin), "focus_out_event", GTK_SIGNAL_FUNC(playlistwin_focus_out), NULL);
 	gtk_signal_connect(GTK_OBJECT(playlistwin), "configure_event", GTK_SIGNAL_FUNC(playlistwin_configure), NULL);
 	gtk_signal_connect(GTK_OBJECT(playlistwin), "client_event", GTK_SIGNAL_FUNC(playlistwin_client_event), NULL);
-	gtk_drag_dest_set(playlistwin, GTK_DEST_DEFAULT_ALL, drop_types, 1, GDK_ACTION_COPY);
-	gtk_signal_connect(GTK_OBJECT(playlistwin), "drag_data_received", GTK_SIGNAL_FUNC(playlistwin_drag_data_received), NULL);
+	xmms_drag_dest_set(playlistwin);
+	gtk_signal_connect(GTK_OBJECT(playlistwin), "drag-data-received", GTK_SIGNAL_FUNC(playlistwin_drag_data_received), NULL);
 	gtk_signal_connect(GTK_OBJECT(playlistwin), "key-press-event", GTK_SIGNAL_FUNC(playlistwin_keypress), NULL);
 
 	gdk_window_set_decorations(playlistwin->window, 0);
@@ -1662,7 +1514,7 @@ void playlistwin_show(void)
 	draw_playlist_window(TRUE);
 	tbutton_set_toggled(mainwin_pl, TRUE);
 	cfg.playlist_visible = TRUE;
-	GTK_CHECK_MENU_ITEM(gtk_item_factory_get_widget(mainwin_general_menu, "/Playlist Editor"))->active = TRUE;
+	GTK_CHECK_MENU_ITEM(gtk_item_factory_get_widget(mainwin_general_menu, _("/Playlist Editor")))->active = TRUE;
 	playlistwin_set_toprow(0);
 	playlist_check_pos_current();
 	mainwin_set_always_on_top(cfg.always_on_top);
@@ -1675,5 +1527,5 @@ void playlistwin_hide(void)
 	gtk_widget_hide(playlistwin);
 	cfg.playlist_visible = FALSE;
 	tbutton_set_toggled(mainwin_pl, FALSE);
-	GTK_CHECK_MENU_ITEM(gtk_item_factory_get_widget(mainwin_general_menu, "/Playlist Editor"))->active = FALSE;
+	GTK_CHECK_MENU_ITEM(gtk_item_factory_get_widget(mainwin_general_menu, _("/Playlist Editor")))->active = FALSE;
 }
