@@ -1,5 +1,6 @@
 /*  XMMS - Cross-platform multimedia player
- *  Copyright (C) 1998-1999  Peter Alm, Mikael Alm, Olle Hallnas, Thomas Nilsson and 4Front Technologies
+ *  Copyright (C) 1998-2000  Peter Alm, Mikael Alm, Olle Hallnas, Thomas Nilsson and 4Front Technologies
+ *  Copyright (C) 1999,2000  Hvard Kvlen
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -24,7 +25,6 @@
 #include "defskin/volume.xpm"
 #include "defskin/monoster.xpm"
 #include "defskin/playpaus.xpm"
-#include "defskin/numbers.xpm"
 #include "defskin/nums_ex.xpm"
 #include "defskin/posbar.xpm"
 #include "defskin/pledit.xpm"
@@ -35,7 +35,7 @@
 
 Skin *skin;
 
-static gint skin_default_viscolor[24][3] =
+static const gint skin_default_viscolor[24][3] =
 {
 	{9,34,53},
 	{10,18,26},
@@ -61,23 +61,16 @@ static gint skin_default_viscolor[24][3] =
 	{0,56,112},
 	{0,32,64},
 	{200, 200, 200}
-
 };
 
 static void setup_skin_masks(void)
 {
 	if (cfg.player_visible)
 	{
-		if (cfg.doublesize)
-			gtk_widget_shape_combine_mask(mainwin, get_skin_pixmap(cfg.player_shaded ? SKIN_MASK_SHADE_DS : SKIN_MASK_MAIN_DS), 0, 0);
-		else
-			gtk_widget_shape_combine_mask(mainwin, get_skin_pixmap(cfg.player_shaded ? SKIN_MASK_SHADE : SKIN_MASK_MAIN), 0, 0);
+		gtk_widget_shape_combine_mask(mainwin, skin_get_mask(SKIN_MASK_MAIN, cfg.doublesize, cfg.player_shaded), 0, 0);
 	}
-	if (cfg.eq_doublesize_linked && cfg.doublesize)
-		gtk_widget_shape_combine_mask(equalizerwin, get_skin_pixmap(cfg.equalizer_shaded ? SKIN_MASK_EQ_SHADE_DS : SKIN_MASK_EQ_DS), 0, 0);
-	else
-		gtk_widget_shape_combine_mask(equalizerwin, get_skin_pixmap(cfg.equalizer_shaded ? SKIN_MASK_EQ_SHADE : SKIN_MASK_EQ), 0, 0);
 
+	gtk_widget_shape_combine_mask(equalizerwin, skin_get_mask(SKIN_MASK_EQ, EQUALIZER_DOUBLESIZE, cfg.equalizer_shaded), 0, 0);
 }
 
 static GdkBitmap *create_default_mask(GdkWindow * parent, gint w, gint h)
@@ -96,42 +89,109 @@ static GdkBitmap *create_default_mask(GdkWindow * parent, gint w, gint h)
 	return ret;
 }
 
+static void load_def_pixmap(SkinPixmap *skinpixmap, gchar **skindata)
+{
+	skinpixmap->def_pixmap = gdk_pixmap_create_from_xpm_d(mainwin->window, NULL, NULL, skindata);
+	gdk_window_get_size(skinpixmap->def_pixmap, &skinpixmap->width, &skinpixmap->height);
+}
+
+static void skin_query_color(GdkColormap *cm, GdkColor *c)
+{
+	XColor xc = {0};
+	
+	xc.pixel = c->pixel;
+	XQueryColor(GDK_COLORMAP_XDISPLAY(cm), GDK_COLORMAP_XCOLORMAP(cm), &xc);
+	c->red = xc.red;
+	c->green = xc.green;
+	c->blue = xc.blue;
+}
+
+static glong skin_calc_luminance(GdkColor *c)
+{
+	return (0.212671 * c->red + 0.715160 * c->green + 0.072169 * c->blue);
+}
+
+static void skin_get_textcolors(GdkPixmap *text, GdkColor *bgc, GdkColor *fgc)
+{
+	/*
+	 * Try to extract reasonable background and foreground colors
+	 * from the font pixmap
+	 */
+	
+	GdkImage *gi;
+	GdkColormap *cm;
+	int i;
+
+	if (text == NULL)
+		return;
+	
+	/* Get the first line of text */
+	gi = gdk_image_get(text, 0, 0, 155, 6);
+	cm = gdk_window_get_colormap(playlistwin->window);
+	for (i = 0; i < 6; i ++)
+	{
+		GdkColor c;
+		gint x;
+		glong d, max_d;
+
+		/* Get a pixel from the middle of the space character */
+		bgc[i].pixel = gdk_image_get_pixel(gi, 151, i);
+		skin_query_color(cm, &bgc[i]);
+
+		max_d = 0;
+		for (x = 1; x < 150; x ++)
+		{
+			c.pixel = gdk_image_get_pixel(gi, x, i);
+			skin_query_color(cm, &c);
+
+			d = labs(skin_calc_luminance(&c) -
+				 skin_calc_luminance(&bgc[i]));
+			if (d > max_d)
+			{
+				memcpy(&fgc[i], &c, sizeof(GdkColor));
+				max_d = d;
+			}
+		}
+	}
+}
+
 void init_skins(void)
 {
 	gint i;
 
 	skin = (Skin *) g_malloc0(sizeof (Skin));
-	skin->def_main = gdk_pixmap_create_from_xpm_d(mainwin->window, NULL, NULL, skin_main);
-	skin->def_cbuttons = gdk_pixmap_create_from_xpm_d(mainwin->window, NULL, NULL, skin_cbuttons);
-	skin->def_titlebar = gdk_pixmap_create_from_xpm_d(mainwin->window, NULL, NULL, skin_titlebar);
-	skin->def_shufrep = gdk_pixmap_create_from_xpm_d(mainwin->window, NULL, NULL, skin_shufrep);
-	skin->def_text = gdk_pixmap_create_from_xpm_d(mainwin->window, NULL, NULL, skin_text);
-	skin->def_volume = gdk_pixmap_create_from_xpm_d(mainwin->window, NULL, NULL, skin_volume);
-	skin->def_monostereo = gdk_pixmap_create_from_xpm_d(mainwin->window, NULL, NULL, skin_monoster);
-	skin->def_playpause = gdk_pixmap_create_from_xpm_d(mainwin->window, NULL, NULL, skin_playpaus);
-	skin->def_numbers = gdk_pixmap_create_from_xpm_d(mainwin->window, NULL, NULL, skin_numbers);
-	skin->def_nums_ex = gdk_pixmap_create_from_xpm_d(mainwin->window, NULL, NULL, skin_nums_ex);
-	skin->def_posbar = gdk_pixmap_create_from_xpm_d(mainwin->window, NULL, NULL, skin_posbar);
-	skin->def_pledit = gdk_pixmap_create_from_xpm_d(mainwin->window, NULL, NULL, skin_pledit);
-	skin->def_eqmain = gdk_pixmap_create_from_xpm_d(mainwin->window, NULL, NULL, skin_eqmain);
-	skin->def_eq_ex = gdk_pixmap_create_from_xpm_d(mainwin->window, NULL, NULL, skin_eq_ex);
+	load_def_pixmap(&skin->main, skin_main);
+	load_def_pixmap(&skin->cbuttons, skin_cbuttons);
+	load_def_pixmap(&skin->titlebar, skin_titlebar);
+	load_def_pixmap(&skin->shufrep, skin_shufrep);
+	load_def_pixmap(&skin->text, skin_text);
+	skin_get_textcolors(skin->text.def_pixmap, skin->def_textbg, skin->def_textfg);
+	load_def_pixmap(&skin->volume, skin_volume);
+	load_def_pixmap(&skin->balance, skin_volume);
+	load_def_pixmap(&skin->monostereo, skin_monoster);
+	load_def_pixmap(&skin->playpause, skin_playpaus);
+	load_def_pixmap(&skin->numbers, skin_nums_ex);
+	load_def_pixmap(&skin->posbar, skin_posbar);
+	load_def_pixmap(&skin->pledit, skin_pledit);
+	load_def_pixmap(&skin->eqmain, skin_eqmain);
+	load_def_pixmap(&skin->eq_ex, skin_eq_ex);
 
 	skin->def_pledit_normal.red = 0x2400;
 	skin->def_pledit_normal.green = 0x9900;
 	skin->def_pledit_normal.blue = 0xffff;
-	gdk_color_alloc(gdk_colormap_get_system(), &skin->def_pledit_normal);
+	gdk_color_alloc(gdk_window_get_colormap(playlistwin->window), &skin->def_pledit_normal);
 	skin->def_pledit_current.red = 0xffff;
 	skin->def_pledit_current.green = 0xee00;
 	skin->def_pledit_current.blue = 0xffff;
-	gdk_color_alloc(gdk_colormap_get_system(), &skin->def_pledit_current);
+	gdk_color_alloc(gdk_window_get_colormap(playlistwin->window), &skin->def_pledit_current);
 	skin->def_pledit_normalbg.red = 0x0A00;
 	skin->def_pledit_normalbg.green = 0x1200;
 	skin->def_pledit_normalbg.blue = 0x0A00;
-	gdk_color_alloc(gdk_colormap_get_system(), &skin->def_pledit_normalbg);
+	gdk_color_alloc(gdk_window_get_colormap(playlistwin->window), &skin->def_pledit_normalbg);
 	skin->def_pledit_selectedbg.red = 0x0A00;
 	skin->def_pledit_selectedbg.green = 0x1200;
 	skin->def_pledit_selectedbg.blue = 0x4A00;
-	gdk_color_alloc(gdk_colormap_get_system(), &skin->def_pledit_selectedbg);
+	gdk_color_alloc(gdk_window_get_colormap(playlistwin->window), &skin->def_pledit_selectedbg);
 	for (i = 0; i < 24; i++)
 	{
 		skin->vis_color[i][0] = skin_default_viscolor[i][0];
@@ -148,18 +208,25 @@ void init_skins(void)
 	create_skin_window();
 }
 
-static guchar hex_char_to_int(gchar c)
+static guint hex_chars_to_int(gchar c, gchar d)
 {
-	if(c>= '0' && c <= '9')
-		return c - '0';
-	return 10 + (toupper(c) - 'A');
+	/*
+	 * Converts a value in the range 0x00-0xFF
+	 * to a integer in the range 0-65535
+	 */
+	gchar str[3];
+
+	str[0] = c;
+	str[1] = d;
+	str[2] = '\0';
+
+	return (CLAMP(strtol(str, NULL, 16), 0, 255) * 256);
 }
 
 GdkColor *load_skin_color(const gchar * path, const gchar * file, const gchar * section, const gchar * key)
 {
 	gchar *filename, *value;
 	GdkColor *color = NULL;
-	gint base = 0;
 
 	filename = find_file_recursively(path, file);
 	if (filename)
@@ -167,17 +234,36 @@ GdkColor *load_skin_color(const gchar * path, const gchar * file, const gchar *
 		value = read_ini_string(filename, section, key);
 		if (value)
 		{
-			color = g_malloc(sizeof (GdkColor));
+			gchar *ptr = value;
+			gint len;
+			color = g_malloc0(sizeof (GdkColor));
 			g_strchug(g_strchomp(value));
 			if (value[0] == '#')
-				base++;
-			else
-				base = 0;
-			color->red = (hex_char_to_int(value[base]) << 12 | hex_char_to_int(value[base] + 1) << 8) | 0xFF;
-			color->green = (hex_char_to_int(value[base + 2]) << 12 | hex_char_to_int(value[base + 3] + 1) << 8) | 0xFF;
-			color->blue = (hex_char_to_int(value[base + 4]) << 12 | hex_char_to_int(value[base + 5] + 1) << 8) | 0xFF;
+				ptr++;
+			len = strlen(ptr);
 
-			gdk_color_alloc(gdk_colormap_get_system(), color);
+			/*
+			 * The handling of incomplete values is done this way
+			 * to maximize winamp compatibility
+			 */
+			if (len >= 6)
+			{
+				color->red = hex_chars_to_int(*ptr,
+							      *(ptr + 1));
+				ptr += 2;
+			}
+			if (len >= 4)
+			{
+				color->green = hex_chars_to_int(*ptr,
+								*(ptr + 1));
+				ptr += 2;
+			}
+			if (len >= 2)
+				color->blue = hex_chars_to_int(*ptr,
+							       *(ptr + 1));
+				
+
+			gdk_color_alloc(gdk_window_get_colormap(playlistwin->window), color);
 			g_free(value);
 		}
 		g_free(filename);
@@ -185,35 +271,27 @@ GdkColor *load_skin_color(const gchar * path, const gchar * file, const gchar *
 	return color;
 }
 
-GdkPixmap *load_skin_pixmap(const gchar * path, const gchar * file, GdkPixmap *def_pixmap)
+static void load_skin_pixmap(SkinPixmap *skinpixmap,
+			     const gchar * path, const gchar * file)
 {
 	char *filename;
-	GdkPixmap *pixmap = NULL, *ret = NULL;
-	gint w,h, pw, ph;
-	static GdkGC *skin_gc = NULL;
+	gint w, h;
 
 	filename = find_file_recursively(path, file);
-	if (filename)
-	{
-		pixmap = read_bmp(filename);
-		if(!pixmap)
-			return NULL;
-		gdk_window_get_size(def_pixmap, &w, &h);
-		ret = gdk_pixmap_new(mainwin->window, w, h,
-				     gdk_rgb_get_visual()->depth);
-		if(!skin_gc)
-			skin_gc = gdk_gc_new(mainwin->window);
-		gdk_draw_pixmap(ret, skin_gc, def_pixmap, 0, 0, 0, 0, w, h);
-		gdk_window_get_size(pixmap, &pw, &ph);
-		if(pw > w)
-			pw = w;
-		if(ph > h)
-			ph = h;
-		gdk_draw_pixmap(ret, skin_gc, pixmap, 0, 0, 0, 0, pw, ph);
-		gdk_pixmap_unref(pixmap);		
-		g_free(filename);
-	}
-	return ret;
+
+	if (!filename)
+		return;
+
+	skinpixmap->pixmap = read_bmp(filename);
+
+	g_free(filename);
+
+	if (!skinpixmap->pixmap)
+		return;
+	gdk_window_get_size(skinpixmap->pixmap, &w, &h);
+
+	skinpixmap->current_width = MIN(w, skinpixmap->width);
+	skinpixmap->current_height = MIN(h, skinpixmap->height);
 }
 
 GdkBitmap *skin_create_transparent_mask(const gchar * path, const gchar * file, const gchar * section, GdkWindow * window, gint width, gint height, gboolean doublesize)
@@ -225,55 +303,58 @@ GdkBitmap *skin_create_transparent_mask(const gchar * path, const gchar * file,
 	GdkColor pattern;
 	GdkPoint *gpoints;
 
-	gboolean created_mask = FALSE, inited_mask = FALSE;
+	gboolean created_mask = FALSE;
 	GArray *num, *point;
 	gint i, j, k;
 
-	if (path)
+	if (!path)
+		return NULL;
+	filename = find_file_recursively(path, file);
+	if (!filename)
+		return NULL;
+
+	if ((num = read_ini_array(filename, section, "NumPoints")) == NULL)
 	{
-		filename = find_file_recursively(path, file);
-		if (filename)
+		g_free(filename);
+		return NULL;
+	}
+		
+	if ((point = read_ini_array(filename, section, "PointList")) == NULL)
+	{
+		g_array_free(num, TRUE);
+		g_free(filename);
+		return NULL;
+	}
+
+	mask = gdk_pixmap_new(window, width, height, 1);
+	gc = gdk_gc_new(mask);
+	
+	pattern.pixel = 0;
+	gdk_gc_set_foreground(gc, &pattern);
+	gdk_draw_rectangle(mask, gc, TRUE, 0, 0, width, height);
+	pattern.pixel = 1;
+	gdk_gc_set_foreground(gc, &pattern);
+		
+	j = 0;
+	for (i = 0; i < num->len; i++)
+	{
+		if ((point->len - j) >= (g_array_index(num, gint, i) * 2))
 		{
-			if ((num = read_ini_array(filename, section, "NumPoints")) != NULL)
+			created_mask = TRUE;
+			gpoints = g_malloc(g_array_index(num, gint, i) * sizeof (GdkPoint));
+			for (k = 0; k < g_array_index(num, gint, i); k++)
 			{
-				if ((point = read_ini_array(filename, section, "PointList")) != NULL)
-				{
-					mask = gdk_pixmap_new(window, width, height, 1);
-					gc = gdk_gc_new(mask);
-
-					pattern.pixel = 0;
-					gdk_gc_set_foreground(gc, &pattern);
-					gdk_draw_rectangle(mask, gc, TRUE, 0, 0, width, height);
-					pattern.pixel = 1;
-					gdk_gc_set_foreground(gc, &pattern);
-					inited_mask = TRUE;
-
-					j = 0;
-					for (i = 0; i < num->len; i++)
-					{
-						if ((point->len - j) >= (g_array_index(num, gint, i) * 2))
-						{
-							created_mask = TRUE;
-							gpoints = g_malloc(g_array_index(num, gint, i) * sizeof (GdkPoint));
-							for (k = 0; k < g_array_index(num, gint, i); k++)
-							{
-								gpoints[k].x = g_array_index(point, gint, j + k * 2) * (1 + doublesize);
-								gpoints[k].y = g_array_index(point, gint, j + k * 2 + 1) * (1 + doublesize);
-							}
-							j += k * 2;
-							gdk_draw_polygon(mask, gc, TRUE, gpoints, g_array_index(num, gint, i));
-							g_free(gpoints);
-						}
-					}
-					g_array_free(num, TRUE);
-				}
-				g_array_free(point, TRUE);
+				gpoints[k].x = g_array_index(point, gint, j + k * 2) * (1 + doublesize);
+				gpoints[k].y = g_array_index(point, gint, j + k * 2 + 1) * (1 + doublesize);
 			}
-			g_free(filename);
+			j += k * 2;
+			gdk_draw_polygon(mask, gc, TRUE, gpoints, g_array_index(num, gint, i));
+			g_free(gpoints);
 		}
 	}
-	if (!inited_mask)
-		return NULL;
+	g_array_free(num, TRUE);
+	g_array_free(point, TRUE);
+	g_free(filename);
 
 	if (!created_mask)
 		gdk_draw_rectangle(mask, gc, TRUE, 0, 0, width, height);
@@ -298,64 +379,87 @@ void load_skin_viscolor(const gchar * path, const gchar * file)
 	}
 
 	filename = find_file_recursively(path, file);
-	if (filename)
-	{
-		if ((f = fopen(filename, "r")) != NULL)
-		{
-			for (i = 0; i < 24; i++)
-			{
-				if (fgets(line, 255, f))
-				{
-					a = string_to_garray(line);
-					if (a->len > 2)
-					{
-						for (c = 0; c < 3; c++)
-							skin->vis_color[i][c] = g_array_index(a, gint, c);
-					}
-					g_array_free(a, TRUE);
-				}
-				else
-					break;
+	if (!filename)
+		return;
 
-			}
-			fclose(f);
-		}
+	if ((f = fopen(filename, "r")) == NULL)
+	{
 		g_free(filename);
+		return;
 	}
+
+	for (i = 0; i < 24; i++)
+	{
+		if (fgets(line, 255, f))
+		{
+			a = string_to_garray(line);
+			if (a->len > 2)
+			{
+				for (c = 0; c < 3; c++)
+					skin->vis_color[i][c] = g_array_index(a, gint, c);
+			}
+			g_array_free(a, TRUE);
+		}
+		else
+			break;
+		
+	}
+	fclose(f);
+	g_free(filename);
+}
+
+static void skin_numbers_generate_dash(SkinPixmap *numbers)
+{
+	GdkGC *gc;
+	GdkPixmap *pixmap;
+
+	if (numbers->pixmap == NULL ||
+	    numbers->current_width < 99)
+		return;
+
+	gc = gdk_gc_new(numbers->pixmap);
+	pixmap = gdk_pixmap_new(mainwin->window, 108,
+				numbers->current_height,
+				gdk_rgb_get_visual()->depth);
+	skin_draw_pixmap(pixmap, gc, SKIN_NUMBERS,
+			 0, 0, 0, 0, 99, 13);
+	skin_draw_pixmap(pixmap, gc, SKIN_NUMBERS,
+			 90, 0, 99, 0, 9, 13);
+	skin_draw_pixmap(pixmap, gc, SKIN_NUMBERS,
+			 20, 6, 101, 6, 5, 1);
+	gdk_gc_unref(gc);
+	gdk_pixmap_unref(numbers->pixmap);
+	numbers->pixmap = pixmap;
+	numbers->current_width = 108;
+}
+
+
+static void skin_free_pixmap(SkinPixmap *p)
+{
+	if (p->pixmap)
+		gdk_pixmap_unref(p->pixmap);
+	p->pixmap = NULL;
 }
 
 void free_skin(void)
 {
 	gint i;
 
-	if (skin->main)
-		gdk_pixmap_unref(skin->main);
-	if (skin->cbuttons)
-		gdk_pixmap_unref(skin->cbuttons);
-	if (skin->titlebar)
-		gdk_pixmap_unref(skin->titlebar);
-	if (skin->shufrep)
-		gdk_pixmap_unref(skin->shufrep);
-	if (skin->text)
-		gdk_pixmap_unref(skin->text);
-	if (skin->volume)
-		gdk_pixmap_unref(skin->volume);
-	if (skin->balance)
-		gdk_pixmap_unref(skin->balance);
-	if (skin->monostereo)
-		gdk_pixmap_unref(skin->monostereo);
-	if (skin->playpause)
-		gdk_pixmap_unref(skin->playpause);
-	if (skin->numbers)
-		gdk_pixmap_unref(skin->numbers);
-	if (skin->posbar)
-		gdk_pixmap_unref(skin->posbar);
-	if (skin->pledit)
-		gdk_pixmap_unref(skin->pledit);
-	if (skin->eqmain)
-		gdk_pixmap_unref(skin->eqmain);
-	if (skin->eq_ex)
-		gdk_pixmap_unref(skin->eq_ex);
+	skin_free_pixmap(&skin->main);
+	skin_free_pixmap(&skin->cbuttons);
+	skin_free_pixmap(&skin->titlebar);
+	skin_free_pixmap(&skin->shufrep);
+	skin_free_pixmap(&skin->text);
+	skin_free_pixmap(&skin->volume);
+	skin_free_pixmap(&skin->balance);
+	skin_free_pixmap(&skin->monostereo);
+	skin_free_pixmap(&skin->playpause);
+	skin_free_pixmap(&skin->numbers);
+	skin_free_pixmap(&skin->posbar);
+	skin_free_pixmap(&skin->pledit);
+	skin_free_pixmap(&skin->eqmain);
+	skin_free_pixmap(&skin->eq_ex);
+
 	if (skin->mask_main)
 		gdk_bitmap_unref(skin->mask_main);
 	if (skin->mask_main_ds)
@@ -372,32 +476,7 @@ void free_skin(void)
 		gdk_bitmap_unref(skin->mask_eq_shade);
 	if (skin->mask_eq_shade_ds)
 		gdk_bitmap_unref(skin->mask_eq_shade_ds);
-	if (skin->pledit_normal)
-		g_free(skin->pledit_normal);
-	if (skin->pledit_current)
-		g_free(skin->pledit_current);
-	if (skin->pledit_normalbg)
-		g_free(skin->pledit_normalbg);
-	if (skin->pledit_selectedbg)
-		g_free(skin->pledit_selectedbg);
-	skin->main = NULL;
-	skin->cbuttons = NULL;
-	skin->titlebar = NULL;
-	skin->shufrep = NULL;
-	skin->text = NULL;
-	skin->volume = NULL;
-	skin->balance = NULL;
-	skin->monostereo = NULL;
-	skin->playpause = NULL;
-	skin->numbers = NULL;
-	skin->posbar = NULL;
-	skin->pledit = NULL;
-	skin->eqmain = NULL;
-	skin->eq_ex = NULL;
-	skin->pledit_normal = NULL;
-	skin->pledit_current = NULL;
-	skin->pledit_normalbg = NULL;
-	skin->pledit_selectedbg = NULL;
+
 	skin->mask_main = NULL;
 	skin->mask_main_ds = NULL;
 	skin->mask_shade = NULL;
@@ -406,6 +485,20 @@ void free_skin(void)
 	skin->mask_eq_ds = NULL;
 	skin->mask_eq_shade = NULL;
 	skin->mask_eq_shade_ds = NULL;
+
+	if (skin->pledit_normal)
+		g_free(skin->pledit_normal);
+	if (skin->pledit_current)
+		g_free(skin->pledit_current);
+	if (skin->pledit_normalbg)
+		g_free(skin->pledit_normalbg);
+	if (skin->pledit_selectedbg)
+		g_free(skin->pledit_selectedbg);
+
+	skin->pledit_normal = NULL;
+	skin->pledit_current = NULL;
+	skin->pledit_normalbg = NULL;
+	skin->pledit_selectedbg = NULL;
 	for (i = 0; i < 24; i++)
 	{
 		skin->vis_color[i][0] = skin_default_viscolor[i][0];
@@ -418,7 +511,7 @@ static void _load_skin(const gchar * path, gboolean force)
 {
 	char *ending, *tmp = NULL, *tempdir, *unzip, *tar;
 
-	if(!force)
+	if (!force)
 	{
 		if (skin->path && path)
 			if (!strcmp(skin->path, path))
@@ -429,7 +522,7 @@ static void _load_skin(const gchar * path, gboolean force)
 	}
 	if (path)
 	{
-		if (strncmp(path, P_tmpdir, strlen(P_tmpdir)))
+		if (strncmp(path, g_get_tmp_dir(), strlen(g_get_tmp_dir())))
 		{
 			skin->path = g_realloc(skin->path, strlen(path) + 1);
 			strcpy(skin->path, path);
@@ -477,22 +570,28 @@ static void _load_skin(const gchar * path, gboolean force)
 				return;
 			}
 		}
-		skin->main = load_skin_pixmap(path, "main.bmp", skin->def_main);
-		skin->cbuttons = load_skin_pixmap(path, "cbuttons.bmp", skin->def_cbuttons);
-		skin->titlebar = load_skin_pixmap(path, "titlebar.bmp", skin->def_titlebar);
-		skin->shufrep = load_skin_pixmap(path, "shufrep.bmp", skin->def_shufrep);
-		skin->text = load_skin_pixmap(path, "text.bmp", skin->def_text);
-		skin->volume = load_skin_pixmap(path, "volume.bmp", skin->def_volume);
-		skin->balance = load_skin_pixmap(path, "balance.bmp", skin->def_volume);
-		skin->monostereo = load_skin_pixmap(path, "monoster.bmp", skin->def_monostereo);
-		skin->playpause = load_skin_pixmap(path, "playpaus.bmp", skin->def_playpause);
-		skin->numbers = load_skin_pixmap(path, "nums_ex.bmp", skin->def_nums_ex);
-		if (!skin->numbers)
-			skin->numbers = load_skin_pixmap(path, "numbers.bmp", skin->def_numbers);
-		skin->posbar = load_skin_pixmap(path, "posbar.bmp", skin->def_posbar);
-		skin->pledit = load_skin_pixmap(path, "pledit.bmp", skin->def_pledit);
-		skin->eqmain = load_skin_pixmap(path, "eqmain.bmp", skin->def_eqmain);
-		skin->eq_ex = load_skin_pixmap(path, "eq_ex.bmp", skin->def_eq_ex);
+		load_skin_pixmap(&skin->main, path, "main.bmp");
+		load_skin_pixmap(&skin->cbuttons, path, "cbuttons.bmp");
+		load_skin_pixmap(&skin->titlebar, path, "titlebar.bmp");
+		load_skin_pixmap(&skin->shufrep, path, "shufrep.bmp");
+		load_skin_pixmap(&skin->text, path, "text.bmp");
+		skin_get_textcolors(skin->text.pixmap, skin->textbg, skin->textfg);
+		load_skin_pixmap(&skin->volume, path, "volume.bmp");
+		load_skin_pixmap(&skin->balance, path, "balance.bmp");
+		if (skin->balance.pixmap == NULL)
+			load_skin_pixmap(&skin->balance, path, "volume.bmp");
+		load_skin_pixmap(&skin->monostereo, path, "monoster.bmp");
+		load_skin_pixmap(&skin->playpause, path, "playpaus.bmp");
+		load_skin_pixmap(&skin->numbers, path, "nums_ex.bmp");
+		if (skin->numbers.pixmap == NULL)
+		{
+			load_skin_pixmap(&skin->numbers, path, "numbers.bmp");
+			skin_numbers_generate_dash(&skin->numbers);
+		}
+		load_skin_pixmap(&skin->posbar, path, "posbar.bmp");
+		load_skin_pixmap(&skin->pledit, path, "pledit.bmp");
+		load_skin_pixmap(&skin->eqmain, path, "eqmain.bmp");
+		load_skin_pixmap(&skin->eq_ex, path, "eq_ex.bmp");
 		skin->pledit_normal = load_skin_color(path, "pledit.txt", "text", "normal");
 		skin->pledit_current = load_skin_color(path, "pledit.txt", "text", "current");
 		skin->pledit_normalbg = load_skin_color(path, "pledit.txt", "text", "normalbg");
@@ -538,121 +637,115 @@ void cleanup_skins(void)
 	free_skin();
 }
 
-GdkPixmap *get_skin_pixmap(SkinIndex si)
+static SkinPixmap *get_skin_pixmap(SkinIndex si)
 {
-	GdkPixmap *ret = NULL;
-
 	switch (si)
 	{
 		case SKIN_MAIN:
-			ret = skin->main;
-			if (!ret)
-				ret = skin->def_main;
-			break;
+			return &skin->main;
 		case SKIN_CBUTTONS:
-			ret = skin->cbuttons;
-			if (!ret)
-				ret = skin->def_cbuttons;
-			break;
+			return &skin->cbuttons;
 		case SKIN_TITLEBAR:
-			ret = skin->titlebar;
-			if (!ret)
-				ret = skin->def_titlebar;
-			break;
+			return &skin->titlebar;
 		case SKIN_SHUFREP:
-			ret = skin->shufrep;
-			if (!ret)
-				ret = skin->def_shufrep;
-			break;
+			return &skin->shufrep;
 		case SKIN_TEXT:
-			ret = skin->text;
-			if (!ret)
-				ret = skin->def_text;
-			break;
+			return &skin->text;
 		case SKIN_VOLUME:
-			ret = skin->volume;
-			if (!ret)
-				ret = skin->def_volume;
-			break;
+			return &skin->volume;
 		case SKIN_BALANCE:
-			ret = skin->balance;
-			if (!ret)
-				ret = get_skin_pixmap(SKIN_VOLUME);
-			break;
+			return &skin->balance;
 		case SKIN_MONOSTEREO:
-			ret = skin->monostereo;
-			if (!ret)
-				ret = skin->def_monostereo;
-			break;
+			return &skin->monostereo;
 		case SKIN_PLAYPAUSE:
-			ret = skin->playpause;
-			if (!ret)
-				ret = skin->def_playpause;
-			break;
+			return &skin->playpause;
 		case SKIN_NUMBERS:
-			ret = skin->numbers;
-			if (!ret)
-				ret = skin->def_numbers;
-			break;
+			return &skin->numbers;
 		case SKIN_POSBAR:
-			ret = skin->posbar;
-			if (!ret)
-				ret = skin->def_posbar;
-			break;
+			return &skin->posbar;
 		case SKIN_PLEDIT:
-			ret = skin->pledit;
-			if (!ret)
-				ret = skin->def_pledit;
-			break;
+			return &skin->pledit;
 		case SKIN_EQMAIN:
-			ret = skin->eqmain;
-			if (!ret)
-				ret = skin->def_eqmain;
-			break;
+			return &skin->eqmain;
 		case SKIN_EQ_EX:
-			ret = skin->eq_ex;
-			if (!ret)
-				ret = skin->def_eq_ex;
-			break;
+			return &skin->eq_ex;
+	}
+	g_error("Unable to find skin pixmap");
+
+	return NULL;
+}
+
+GdkBitmap* skin_get_mask(MaskIndex mi, gboolean doublesize, gboolean shaded)
+{
+	GdkBitmap *ret = NULL;
+	
+	switch (mi)
+	{
 		case SKIN_MASK_MAIN:
-			ret = skin->mask_main;
-			if (!ret)
-				ret = skin->def_mask;
-			break;
-		case SKIN_MASK_MAIN_DS:
-			ret = skin->mask_main_ds;
-			if (!ret)
-				ret = skin->def_mask_ds;
+			if (!shaded)
+			{
+				if (!doublesize)
+				{
+					ret = skin->mask_main;
+					if (!ret)
+						ret = skin->def_mask;
+				}
+				else
+				{
+					ret = skin->mask_main_ds;
+					if (!ret)
+						ret = skin->def_mask_ds;
+					break;
+				}
+			}
+			else
+			{
+				if (!doublesize)
+				{
+					ret = skin->mask_shade;
+					if (!ret)
+						ret = skin->def_mask_shade;
+				}
+				else
+				{
+					ret = skin->mask_shade_ds;
+					if (!ret)
+						ret = skin->def_mask_shade_ds;
+					break;
+				}
+			}
 			break;
 		case SKIN_MASK_EQ:
-			ret = skin->mask_eq;
-			if (!ret)
-				ret = skin->def_mask;
-			break;
-		case SKIN_MASK_EQ_DS:
-			ret = skin->mask_eq_ds;
-			if (!ret)
-				ret = skin->def_mask_ds;
-			break;
-		case SKIN_MASK_SHADE:
-			ret = skin->mask_shade;
-			if (!ret)
-				ret = skin->def_mask_shade;
-			break;
-		case SKIN_MASK_SHADE_DS:
-			ret = skin->mask_shade_ds;
-			if (!ret)
-				ret = skin->def_mask_shade_ds;
-			break;
-		case SKIN_MASK_EQ_SHADE:
-			ret = skin->mask_eq_shade;
-			if (!ret)
-				ret = skin->def_mask_shade;
-			break;
-		case SKIN_MASK_EQ_SHADE_DS:
-			ret = skin->mask_eq_shade_ds;
-			if (!ret)
-				ret = skin->def_mask_shade_ds;
+			if (!shaded)
+			{
+				if (!doublesize)
+				{
+					ret = skin->mask_eq;
+					if (!ret)
+						ret = skin->def_mask;
+				}
+				else
+				{
+					ret = skin->mask_eq_ds;
+					if (!ret)
+						ret = skin->def_mask_ds;
+				}
+			}
+			else
+			{
+				if (!doublesize)
+				{
+					ret = skin->mask_eq_shade;
+					if (!ret)
+						ret = skin->def_mask_shade;
+				}
+				else
+				{
+					ret = skin->mask_eq_shade_ds;
+					if (!ret)
+						ret = skin->def_mask_shade_ds;
+				}
+			}
 			break;
 	}
 	return ret;
@@ -684,6 +777,18 @@ GdkColor *get_skin_color(SkinColorIndex si)
 			if (!ret)
 				ret = &skin->def_pledit_selectedbg;
 			break;
+	        case SKIN_TEXTBG:
+			if (skin->text.pixmap)
+				ret = skin->textbg;
+			else
+				ret = skin->def_textbg;
+			break;
+	        case SKIN_TEXTFG:
+			if (skin->text.pixmap)
+				ret = skin->textfg;
+			else
+				ret = skin->def_textfg;
+			break;
 	}
 	return ret;
 }
@@ -712,3 +817,46 @@ gboolean is_new_skin(gchar * old_skin)
 		return TRUE;
 	return FALSE;
 }
+
+void skin_draw_pixmap(GdkDrawable *drawable, GdkGC *gc, SkinIndex si,
+		      gint xsrc, gint ysrc, gint xdest, gint ydest,
+		      gint width, gint height)
+{
+	SkinPixmap *pixmap = get_skin_pixmap(si);
+	GdkPixmap *tmp;
+
+	if (pixmap->pixmap != NULL)
+	{
+		if (xsrc > pixmap->current_width || ysrc > pixmap->current_height)
+			return;
+
+		tmp = pixmap->pixmap;
+		width = MIN(width, pixmap->current_width - xsrc);
+		height = MIN(height, pixmap->current_height - ysrc);
+	}
+	else
+		tmp = pixmap->def_pixmap;
+
+	gdk_draw_pixmap(drawable, gc, tmp, xsrc, ysrc, xdest, ydest, width, height);
+}
+
+void skin_get_eq_spline_colors(guint32 (*colors)[19])
+{
+	gint i;
+	GdkPixmap *pixmap;
+	GdkImage *img;
+
+	if (skin->eqmain.pixmap != NULL &&
+	    skin->eqmain.current_width >= 116 &&
+	    skin->eqmain.current_height >= 313)
+		pixmap = skin->eqmain.pixmap;
+	else
+		pixmap = skin->eqmain.def_pixmap;
+
+	img = gdk_image_get(pixmap, 115, 294, 1, 19);
+	
+	for (i = 0; i < 19; i++)
+		(*colors)[i] = gdk_image_get_pixel(img, 0, i);
+
+	gdk_image_destroy(img);
+}
